/***************************************************************************
*
*	File name : extflsc_drv.c
*	Function  : External flash control driver (C17801)
*	Revision  :
*		September. 26, 2007	Yakushiji		First revision
*
****************************************************************************/

// Include File
#include	"reg_801.h"
#include	"system.h"
#include	"extflsc_api.h"


void			ExtFlscInit(T_EXTFLSC_CFG *);
int				ExtFlscErase(unsigned char, unsigned long, unsigned long);
int				ExtFlscWrite(unsigned long, unsigned long, unsigned short*);
static int		CheckBusyVerify(unsigned long, unsigned short);

static unsigned long	ControlReg = 0xFFFFFFFF;

/*************************************************************
	ExtFlscInit()

	description:
		Initialize of External Flash controller.

	argument	: None
	return		: None

**************************************************************/
void ExtFlscInit(T_EXTFLSC_CFG *config){

	ControlReg = config->control;
}

/*************************************************************
	ExtFlscErase()

	description:
		External flash erase.

	argument	:
		mode        : Chip Erase or Sector Erase
		startAddress: Erase start address
		endAddress  : Erase end address

	return		:
		ERROR  : Parameter error or erase error
		SUCCESS: Erase complete

**************************************************************/
int ExtFlscErase(unsigned char mode, unsigned long startAddress, unsigned long endAddress){

	int				ret = ERROR;
	unsigned long	address;

	if(ControlReg == 0xFFFFFFFF){
		return ret;
	}

	// Check start and end address
	if(((startAddress >= EXTFLSC_TOP_ADDRESS && startAddress <= EXTFLSC_END_ADDRESS) ||
	   (endAddress >= EXTFLSC_TOP_ADDRESS && endAddress <= EXTFLSC_END_ADDRESS)) &&
	   (startAddress <= endAddress)){

		address = startAddress;

		while(1){

			// Erase command sequence
			*((volatile unsigned char *)(ControlReg + EXTFLSC_COM1_ADDR)) = EXTFLSC_ERASE1;
			*((volatile unsigned char *)(ControlReg + EXTFLSC_COM2_ADDR)) = EXTFLSC_ERASE2;
			*((volatile unsigned char *)(ControlReg + EXTFLSC_COM1_ADDR)) = EXTFLSC_ERASE3;
			*((volatile unsigned char *)(ControlReg + EXTFLSC_COM1_ADDR)) = EXTFLSC_ERASE4;
			*((volatile unsigned char *)(ControlReg + EXTFLSC_COM2_ADDR)) = EXTFLSC_ERASE5;

			// Check erase type
			if(mode == EXTFLSC_ERASE_CHIP){
				*((volatile unsigned char *)(ControlReg + EXTFLSC_COM1_ADDR)) = EXTFLSC_ERASE_CHIP;
			} else if(mode == EXTFLSC_ERASE_SECTOR){
				// Erase command sequence
				*((volatile unsigned char *)address) = EXTFLSC_ERASE_SECTOR;
			}

			if(CheckBusyVerify(address, 0xFFFF) == ERROR){
				break;
			}

			if(mode == EXTFLSC_ERASE_CHIP){
				ret = SUCCESS;
				break;
			}

			if((address >= 0x7F0000) && (address < 0x810000)){
				address += EXTFLSC_SMALL_SEC;
			} else {
				address += EXTFLSC_LARGE_SEC;
			}

			if(endAddress < address){
				ret = SUCCESS;
				break;
			}

		}
	}

	return ret;
}


/*************************************************************
	ExtFlscWrite()

	description:
		Writing to Flash.

	argument	:
		address : start write address
		size    : data size
		data    : data pointer

	return		:
		ERROR  : Parameter error or write error
		SUCCESS: Writing complete

**************************************************************/
int ExtFlscWrite(unsigned long address, unsigned long size, unsigned short *data){

	int				ret = ERROR;
	unsigned long	i,j;

	if(ControlReg == 0xFFFFFFFF){
		return ret;
	}

	// Check address
	if(address >= EXTFLSC_TOP_ADDRESS && address <= EXTFLSC_END_ADDRESS && ((address & 0x00001) == 0)){

		for(i = j = 0; i < size; i += 2,j++){
			// Program command sequence
			*((volatile unsigned short *)(ControlReg + EXTFLSC_COM1_ADDR)) = EXTFLSC_PROG1;
			*((volatile unsigned short *)(ControlReg + EXTFLSC_COM2_ADDR)) = EXTFLSC_PROG2;
			*((volatile unsigned short *)(ControlReg + EXTFLSC_COM1_ADDR)) = EXTFLSC_PROG3;

			*((volatile unsigned short *)address) = data[j];

			if(CheckBusyVerify(address, data[j]) == ERROR){
				break;
			}

			address += 2;
		}

		if(i == size){
			ret = SUCCESS;
		}
	}

	return ret;
}

/*************************************************************
	CheckBusyVerify()

	description:
		Check busy and data verify.

	argument	:
		address : check address
		data    : verify data

	return		:
		ERROR   : Verify error
		SUCCESS : Normal complete

**************************************************************/
static int	CheckBusyVerify(unsigned long address, unsigned short data)
{
	int				ret = ERROR;	// Return Value
	unsigned short	nowToggle;
	unsigned short	oldToggle;

	oldToggle = 0xFFFF;

	// Toggle bit check
	while(1){
		nowToggle = *((volatile unsigned short *)address) & EXTFLSC_B_DQ6;	// Toggle bit check

		if(oldToggle != nowToggle){
			oldToggle = nowToggle;
		} else {
			break;
		}
	}

	// data check
	if(*((volatile unsigned short *)address) == data){
		ret = SUCCESS;
	}

	return ret;
}

