/***************************************************************************
*
*	File name : usb_bulk.c
*	Function  : USB driver (C17801)
*			Transfer or Receive control for bulk pipe.
*	Revision  :
*		August. 20, 2007	Yakushiji		First revision
*
****************************************************************************/

#include	"reg_801.h"
#include	"system.h"
#include	"usb_api.h"
#include	"usb_request.h"
#include	"usb_bulk.h"

#ifdef	FOR_BULK_TRANSFER

static unsigned long	EPxActualLength[4];		// Endpoint x actual length
static unsigned long	EPxRequestLength[4];	// Endpoint x request length
static unsigned char	*EPxBuffer[4];			// Endpoint x buffer pointer

unsigned char			g_EPxState[4];			// Endpoint x state
unsigned char			g_EPxResult[4];			// Endpoint x result


void			UsbSetEPxParameter(unsigned char, unsigned char *, unsigned long);
unsigned long	UsbEPxRemainData(unsigned char);
void			UsbEPxSetState(unsigned char, unsigned char);
unsigned char	UsbEPxGetState(unsigned char);
void			UsbEPxClearResult(unsigned char);
unsigned char	UsbEPxGetResult(unsigned char);
void			UsbEPxDataSet(unsigned char);
void			UsbEPxDataGet(unsigned char);


/*************************************************************
	UsbSetEPxParameter()

	description:
		Set parameter for endpoint x.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		buffer	: Data buffer pointer
		length	: Data length
	return		: None

**************************************************************/
void UsbSetEPxParameter(unsigned char number, unsigned char *buffer, unsigned long length){

	number--;

	EPxBuffer[number] = buffer;
	EPxRequestLength[number] = EPxActualLength[number] = length;

	g_EPxResult[number] = USB_EPX_NO_RESULT;
}

/*************************************************************
	UsbEPxRemainData()

	description:
		Get remain data length for endpoint x.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		Remain data length

**************************************************************/
unsigned long UsbEPxRemainData(unsigned char number){

	number--;

	return EPxActualLength[number];
}

/*************************************************************
	UsbEPxSetState()

	description:
		Set state for endpoint x.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		state	: set state value
					USB_EPX_DATA_SET
					USB_EPX_DATA_SEND
					USB_EPX_DATA_GET
					USB_EPX_DATA_RCV
					USB_EPX_IDLE
					USB_EPX_STALL
					USB_EPX_RESET
	return		: None

**************************************************************/
void UsbEPxSetState(unsigned char number, unsigned char state){

	number--;

	if(state & USB_EPX_STALL){
		g_EPxState[number] |= state;
	} else {
		g_EPxState[number] = state;
	}

}

/*************************************************************
	UsbEPxGetState()

	description:
		Get state for endpoint x.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		get state value
					USB_EPX_DATA_SET
					USB_EPX_DATA_SEND
					USB_EPX_DATA_GET
					USB_EPX_DATA_RCV
					USB_EPX_IDLE
					USB_EPX_STALL
					USB_EPX_RESET
					USB_EPX_FEATURE

**************************************************************/
unsigned char UsbEPxGetState(unsigned char number){

	number--;

	return g_EPxState[number];
}

/*************************************************************
	UsbEPxClearResult()

	description:
		Clear result for endpoint x.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		: None

**************************************************************/
void UsbEPxClearResult(unsigned char number){

	number--;

	g_EPxResult[number] = USB_EPX_NO_RESULT;
}

/*************************************************************
	UsbEPxGetResult()

	description:
		Get result for endpoint x.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		get state value
					USB_EPX_IN_ERR
					USB_EPX_OUT_ERR
					USB_EPX_IN_ACK
					USB_EPX_OUT_ACK
					USB_EPX_NO_RESULT

**************************************************************/
unsigned char UsbEPxGetResult(unsigned char number){

	number--;

	return g_EPxResult[number];
}

/*************************************************************
	UsbEPxDataSet()

	description:
		Set data for endpoint x.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		: None

**************************************************************/
void UsbEPxDataSet(unsigned char number){

	unsigned char	set;
	unsigned short	length;
	unsigned short	remain;
	unsigned char	num;

	num = number - 1;

	set = 0;

	UsbSetEndpointControl(number, USB_EP_CTL_NAK, 0);

	UsbFIFOWriteRemain(number, &remain);

	if(EPxActualLength[num] >= remain){

		UsbFIFOWrite(number, EPxBuffer[num], remain, &length);

		EPxBuffer[num] += length;
		EPxActualLength[num] -= length;

	} else {

		UsbFIFOWrite(number, EPxBuffer[num], EPxActualLength[num], &length);

		EPxBuffer[num] += length;
		EPxActualLength[num] -= length;

		if(EPxRequestLength[num] & (g_MaxPacket[g_CurrentAlterSet][number] - 1)){
			set = USB_EP_CTL_ENSHORTPKT;
		}
	}

	UsbSetEndpointControl(number, set, USB_EP_CTL_NAK);
}

/*************************************************************
	UsbEPxDataGet()

	description:
		Get data for endpoint x.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		: None

**************************************************************/
void UsbEPxDataGet(unsigned char number){

	unsigned short	remain;
	unsigned short	length;
	unsigned char	num;
	unsigned short	request;

	num = number - 1;

	UsbSetEndpointControl(number, USB_EP_CTL_NAK, 0);

	UsbFIFOReadRemain(number, &remain);

	if(EPxActualLength[num] < remain){
		request = EPxActualLength[num];
		UsbFIFORead(number, EPxBuffer[num], EPxActualLength[num], &length);
	} else {
		request = remain;
		UsbFIFORead(number, EPxBuffer[num], remain, &length);
	}

	EPxBuffer[num] += length;
	EPxActualLength[num] -= length;

	if(EPxActualLength[num] != 0){
		UsbSetEndpointControl(number, 0, USB_EP_CTL_NAK);
	}
}

#endif
