/***************************************************************************
*
*	File name : usb_control.c
*	Function  : USB driver (C17801)
*			Transfer or Receive control for Control pipe.
*	Revision  :
*		August. 20, 2007	Yakushiji		First revision
*
****************************************************************************/

#include	"reg_801.h"
#include	"system.h"
#include	"usb_api.h"
#include	"usb_request.h"
#include	"usb_control.h"

#define	STANDARD_REQUEST		(0x00)
#define	CLASS_REQUEST			(0x20)
#define	VENDOR_REQUEST			(0x40)

T_USB_REQUEST_PACKET	g_RequestPacket;	// Request packet

unsigned char			g_EP0State;			// Endpoint0 state
unsigned char			g_EP0Result;		// Endpoint0 result

static unsigned char	EP0Retry;			// Endponit0 retry counter
static unsigned char	*EP0Buffer;			// Endpoint0 buffer pointer
static unsigned char	CompleteFlag;		// Endpoint0 transaction complete flag

static unsigned long	RequestLength;		// Request length for Request Packet
static unsigned char	FinishFlag;			// Class or Vendor Request transfer finish

void			UsbCtrlProcess(void);
void			UsbSetCtrlParameter(unsigned char *, unsigned short);
int				UsbCtrlRemainData(unsigned short *);
void			UsbCtrlFinish(int);
static void		CtrlRequest(void);
static void		CtrlDataSet(void);
static void		CtrlDataSend(void);
static void		CtrlDataReceive(void);
static void		CtrlDataGet(void);
static void		CtrlStatusStart(void);
static void		CtrlStatusEnd(void);
static void		Idle(void);

// Control function table
static void (*ControlTable[])() = {
	CtrlRequest,
	CtrlDataSet,
	CtrlDataSend,
	CtrlDataReceive,
	CtrlDataGet,
	CtrlStatusStart,
	CtrlStatusEnd,
	Idle
};

/*************************************************************
	UsbCtrlProcess()

	description:
		Set parameter for Control pipe transfer.

	argument	:
		buffer	: Data buffer pointer
		length	: Data length
	return		: None

**************************************************************/
void UsbCtrlProcess(void){

	ControlTable[g_EP0State]();
}

/*************************************************************
	UsbSetCtrlParameter()

	description:
		Set parameter for Control pipe transfer.

	argument	:
		buffer	: Data buffer pointer
		length	: Data length
	return		: None

**************************************************************/
void UsbSetCtrlParameter(unsigned char *buffer, unsigned short length){

	EP0Buffer = buffer;
	g_EP0TotalLength = g_EP0ActualLength = length;

	CompleteFlag = 0;
}

/*************************************************************
	UsbCtrlRemainData()

	description:
		Get remain length for Control pipe.

	argument	: None
	return		:
		Data remain length. if 0 then transfer complete.

**************************************************************/
int UsbCtrlRemainData(unsigned short *remain){

	int		ret = ERROR;

	if(g_EP0ActualLength == 0){
		if(CompleteFlag == 0){
			*remain = 1;
		} else {
			*remain = 0;
			CompleteFlag = 0;
		}
		ret = SUCCESS;
	} else {
		if(CompleteFlag == 1){
			CompleteFlag = 0;
		} else {
			*remain = g_EP0ActualLength;
			ret = SUCCESS;
		}
	}

	return ret;
}

/*************************************************************
	UsbCtrlFinish()

	description:
		Data transfer end for Control pipe.

	argument	:
		end		: SUCCESS
				: ERROR
	return		: None

**************************************************************/
void UsbCtrlFinish(int end){

	if(end == SUCCESS || g_EP0State == EP0_REQ_CHECK){
		FinishFlag = 1;
	} else {
		if(g_EP0State != EP0_IDLE){
			UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_STALL, 0);
			UsbSetEP0Control(USB_EP0_OUT, USB_EP_CTL_STALL, 0);

			g_EP0State = EP0_IDLE;
		}
	}
}

/*************************************************************
	CtrlRequest()

	description:
		Check receive request packet
		for control pipe processing.

	argument	: None
	return		: None

**************************************************************/
static void CtrlRequest(void)
{
	int		ret = ERROR;

	g_ManagementFlag.Init = 0;
	g_ManagementFlag.USBReqRcvd = 0;

	// Clear EP0 FIFO
	USB_EPnControl |= USB_EPnControl_EP0FIFOClr;

	// EnShortPkt = 0
	UsbSetEP0Control(USB_EP0_IN, 0, USB_EP_CTL_ENSHORTPKT);

	// Clear EP0 all interrupt factor
	UsbClearEP0Int(UsbGetEP0Int());

	// Enable Interrupt ACK
	USB_EP0IntEnb |= USB_EP0IntEnb_EnINTranACK;

	// Clear RcvEP0SETUP interrupt factor
	UsbClearMainInt(USB_MainIntStat_RcvEP0SETUP);

	// Get Request packet
	UsbGetSetupPacket(&g_RequestPacket);

	// Set direction
	UsbSetEP0Direction((g_RequestPacket.requestType & 0x80));

	RequestLength = g_RequestPacket.length;

	// check Request
	switch(g_RequestPacket.requestType & 0x60){
	case STANDARD_REQUEST:			// Standard Request
		FinishFlag = 1;

		ret = StandardRequest();
		break;

	case CLASS_REQUEST:				// Class Request
#ifdef	FOR_CLASS_REQUEST
		FinishFlag = 0;

		ret = g_Config.classFunc(&g_RequestPacket);
#else
		FinishFlag = 1;
#endif
		break;

	case VENDOR_REQUEST:			// Vendor Request
		FinishFlag = 0;

		ret = g_Config.vendorFunc(&g_RequestPacket);
		break;

	default:
		break;
	}

	// if Request parameter is OK then change Control pipe state
	if(ret == SUCCESS){

		if(g_EP0State == EP0_REQ_CHECK){
			// Valid Data length
			if(RequestLength != 0){

				// Check direction
				if(g_RequestPacket.requestType & 0x80){
					// Transfer
					g_EP0State = EP0_DATA_SET;
				} else {
					// Receive
					UsbSetEP0Control(USB_EP0_OUT, 0, USB_EP_CTL_NAK);
					g_EP0State = EP0_DATA_GET;
				}
			} else {
				// status stage
				g_EP0State = EP0_STS_START;
			}
		}

	// Request parameter is Error
	} else {
		if(FinishFlag == 1){
			// Control pipe is STALL
			UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_STALL, 0);
			UsbSetEP0Control(USB_EP0_OUT, USB_EP_CTL_STALL, 0);

			g_EP0State = EP0_IDLE;
		}
	}
}

/*************************************************************
	CtrlDataSet()

	description:
		Set Data for Control pipe.

	argument	: None
	return		: None

**************************************************************/
static void CtrlDataSet(void)
{
	unsigned short		length;
	unsigned short		size;
	unsigned char		set;

	if(g_EP0ActualLength != 0){
		// Actual data length is bigger than Max Packet
		if(g_EP0ActualLength >= g_MaxPacket[0][0]){

			size = g_MaxPacket[0][0];
			set = 0;

		} else {

			size = g_EP0ActualLength;
			set = USB_EP_CTL_ENSHORTPKT;
		}

		UsbFIFOWrite(USB_ENDPOINT_0, EP0Buffer, size, &length);

		g_EP0ActualLength -= length;
		EP0Buffer += length;

		g_EP0State  = EP0_DATA_SEND;
		g_EP0Result = EP0_NO_RESULT;

		UsbSetEP0Control(USB_EP0_IN, set, USB_EP_CTL_NAK);
	}
}

/*************************************************************
	CtrlDataSend()

	description:
		Send complete check for control pipe.

	argument	: None
	return		: None

**************************************************************/
static void CtrlDataSend(void)
{
	// IN_TranACK
	if(g_EP0Result == EP0_IN_ACK){

		g_EP0Result = EP0_NO_RESULT;
		EP0Retry = 0;

		// Valid g_EP0ActualLength
		if(g_EP0ActualLength){
			// set Next data
			g_EP0State = EP0_DATA_SET;

		} else {
			// RequestLength Complete
			if(RequestLength == g_EP0TotalLength){
				// change Status state
				g_EP0State = EP0_STS_START;

				// Class/Vendor complete
				CompleteFlag = 1;
			} else {
				// It doesn't divide by g_MaxPacket size.
				if(g_EP0TotalLength & (g_MaxPacket[0][0] - 1)){
					// change status stage
					g_EP0State = EP0_STS_START;

					// Class/Vendor complete
					CompleteFlag = 1;
				} else {
					// send 0 length data
					UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_ENSHORTPKT, 0);
				}
			}
		}
	} else {
		// IN_TranErr
		if(g_EP0Result == EP0_IN_ERR){

			g_EP0Result = EP0_NO_RESULT;

			// Present ForceSTALL
			if((UsbGetEP0Control(USB_EP0_IN) & USB_EP_CTL_STALL) == USB_EP_CTL_STALL){
				Panic();
			}

		} else {
			// Descriptor reply complete
			if(g_EP0Result == DESCRIPTOR_CMP){

				// get remain desctiptor
				g_EP0ActualLength = UsbRemainDescriptor();

				EP0Retry = 0;
				UsbFIFOClear(USB_ENDPOINT_0);

				// Data transmission isn't done.
				if(g_EP0ActualLength == RequestLength){
					// ForceSTALL
					UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_STALL, 0);
					UsbSetEP0Control(USB_EP0_OUT, USB_EP_CTL_STALL, 0);

					g_EP0State = EP0_IDLE;

				} else {
					g_EP0State = EP0_STS_START;
				}
			} else {
				// Receive next request
				if(g_ManagementFlag.USBReqRcvd){
					g_EP0State = EP0_REQ_CHECK;

					// Class/Vendor complete
					CompleteFlag = 1;

				} else {
					// Valid OUT_TranNAK
					if((UsbGetEP0Int() & USB_EP_INT_OUT_TRAN_NAK) == USB_EP_INT_OUT_TRAN_NAK){
						// Clear OUT_TranNAK
						UsbClearEP0Int(USB_EP_INT_OUT_TRAN_NAK);

						EP0Retry++;

						// Retry over
						if(EP0Retry > USB_MAX_RETRY_CNT){
							g_EP0ActualLength = EP0Retry = 0;
							UsbFIFOClear(USB_ENDPOINT_0);

							g_EP0State = EP0_STS_START;

							// Class/Vendor complete
							CompleteFlag = 1;
						}
					}
				}
			}
		}
	}
}

/*************************************************************
	CtrlDataReceive()

	description:
		Receive complete check for control pipe.

	argument	: None
	return		: None

**************************************************************/
static void CtrlDataReceive(void)
{
	// Valid OUT_TranACK
	if(g_EP0Result == EP0_OUT_ACK){

		g_EP0Result = EP0_NO_RESULT;
		EP0Retry = 0;
		g_EP0State = EP0_DATA_GET;
	} else {
		// Valid OUT_TranErr
		if(g_EP0Result == EP0_OUT_ERR){

			// Present ForceSTALL
			if((UsbGetEP0Control(USB_EP0_OUT) & USB_EP_CTL_STALL) == USB_EP_CTL_STALL){
				Panic();
			}

		} else {
			// Receive next request
			if(g_ManagementFlag.USBReqRcvd){
				g_EP0State = EP0_REQ_CHECK;

				// Class/Vendor complete
				CompleteFlag = 1;
			} else {
				// Valid IN_TranNAK
				if((UsbGetEP0Int() & USB_EP_INT_IN_TRAN_NAK) == USB_EP_INT_IN_TRAN_NAK){
					// Clear IN_TranNAK
					UsbClearEP0Int(USB_EP_INT_IN_TRAN_NAK);

					EP0Retry++;

					// Retry over
					if(EP0Retry > USB_MAX_RETRY_CNT){
						g_EP0ActualLength = EP0Retry = 0;

						UsbFIFOClear(USB_ENDPOINT_0);

						g_EP0State = EP0_STS_START;

						// Class/Vendor complete
						CompleteFlag = 1;
					}
				}
			}
		}
	}
}

/*************************************************************
	CtrlDataGet()

	description:
		Get receive data for control pipe.

	argument	: None
	return		: None

**************************************************************/
static void CtrlDataGet(void)
{
	unsigned short	remain;
	unsigned short	actual;

	if(g_EP0ActualLength != 0){

		g_EP0Result = EP0_NO_RESULT;

		UsbFIFOReadRemain(USB_ENDPOINT_0, &remain);

		// FIFO Empty
		if(remain == 0){
			// Valid g_EP0ActualLength
			if(g_EP0ActualLength){
				// Data not received
				if(g_EP0TotalLength == g_EP0ActualLength){
					// change next receive
					g_EP0State = EP0_DATA_RCV;
				} else {
					// ForceSTALL
					UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_STALL, 0);
					UsbSetEP0Control(USB_EP0_OUT, USB_EP_CTL_STALL, 0);

					g_EP0State = EP0_IDLE;

					// Class/Vendor complete
					CompleteFlag = 1;
				}
			} else {
				// complete
				g_EP0State = EP0_STS_START;

				// Class/Vendor complete
				CompleteFlag = 1;
			}
		} else {
			// Not empty

			// It was received more than the size.
			if( g_EP0ActualLength < remain ){
				// ForceSTALL
				UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_STALL, 0);
				UsbSetEP0Control(USB_EP0_OUT, USB_EP_CTL_STALL, 0);

				g_EP0State = EP0_IDLE;

				// Class/Vendor complete
				CompleteFlag = 1;
			} else {

				UsbFIFORead(USB_ENDPOINT_0, EP0Buffer, remain, &actual);

				EP0Buffer += actual;
				g_EP0ActualLength -= actual;

				// Valid g_EP0ActualLength
				if(g_EP0ActualLength){
					// Clear ForceNAK
					UsbSetEP0Control(USB_EP0_OUT, 0, USB_EP_CTL_NAK);

					// change next receive
					g_EP0State = EP0_DATA_RCV;
				} else {
					// complete
					g_EP0State = EP0_STS_START;

					// Class/Vendor complete
					CompleteFlag = 1;
				}
			}
		}
	}
}

/*************************************************************
	CtrlStatusStart()

	description:
		Status stage start process for control pipe.

	argument	: None
	return		: None

**************************************************************/
static void CtrlStatusStart(void)
{
	// Finish check
	if(FinishFlag == 1){
		// Clear all interrupt
		UsbClearEP0Int(UsbGetEP0Int());

		// IN direction
		if(UsbGetEP0Direction() == USB_EP_IN){
			// change OUT direction
			UsbSetEP0Direction(USB_EP_OUT);

			// Clear ForceNAK
			UsbSetEP0Control(USB_EP0_OUT, 0, USB_EP_CTL_NAK);
		} else {
			// change IN direction
			UsbSetEP0Direction(USB_EP_IN);

			// Clear ForceNAK & Set ShortPacket
			UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_ENSHORTPKT, USB_EP_CTL_NAK);
		}

		g_EP0Result = EP0_NO_RESULT;
		g_EP0State  = EP0_STS_END;
	} else {
		if(g_EP0ActualLength != 0){
			// IN direction
			if(UsbGetEP0Direction() == USB_EP_IN){
				g_EP0State = EP0_DATA_SET;
			} else {
				g_EP0State = EP0_DATA_RCV;
				// Clear ForceNAK
				UsbSetEP0Control(USB_EP0_OUT, 0, USB_EP_CTL_NAK);
			}
		}
	}
}

/*************************************************************
	CtrlStatusEnd()

	description:
		Status stage complete check for control pipe.

	argument	: None
	return		: None

**************************************************************/
static void CtrlStatusEnd(void)
{
	unsigned short		remain;

	if(g_EP0Result == EP0_IN_ACK || g_EP0Result == EP0_OUT_ACK){
		/* IN_TranACKOUT_TranACK̎ */

		g_EP0Result = EP0_NO_RESULT;
		EP0Retry = 0;

		UsbFIFOReadRemain(USB_ENDPOINT_0, &remain);

		// Not empty
		if(remain != 0){
			// ForceSTALL
			UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_STALL, 0);
			UsbSetEP0Control(USB_EP0_OUT, USB_EP_CTL_STALL, 0);
		} else {
			// FIFO empty

#ifdef	FOR_TEST_MODE
			// Request Test Mode
			if(g_ManagementFlag.TestMode){
				g_ManagementFlag.TestMode = 0;

				// Start test mode
				UsbStartTestMode();
			}
#endif

#ifdef	FOR_ADDRESS
			if(g_ManagementFlag.SetAddress && g_Config.addressMode == USB_ADDRESS_MANUAL){
				g_ManagementFlag.SetAddress = 0;

				UsbSetAddress(g_RequestPacket.value);
				// change Address state
				g_UsbState = USB_ADDRESS_STATE;
			}
#endif

		}

		g_EP0State = EP0_IDLE;

	} else {
		// IN_TranErr ot OUT_TranErr
		if(g_EP0Result == EP0_IN_ERR || g_EP0Result == EP0_OUT_ERR){

			g_EP0Result = EP0_NO_RESULT;

			if(g_EP0Result == EP0_IN_ERR){
				// Present ForceSTALL
				if((UsbGetEP0Control(USB_EP0_IN) & USB_EP_CTL_STALL) == USB_EP_CTL_STALL){
					Panic();
				}
			} else {
				// Present ForceSTALL
				if((UsbGetEP0Control(USB_EP0_OUT) & USB_EP_CTL_STALL) == USB_EP_CTL_STALL){
					Panic();
				}
			}
		} else {
			// Receive next request
			if(g_ManagementFlag.USBReqRcvd){
				g_EP0State = EP0_REQ_CHECK;
			} else {
				// IN direction
				if(UsbGetEP0Direction() == USB_EP_IN){

					// Valid OUT_TranNAK
					if((UsbGetEP0Int() & USB_EP_INT_OUT_TRAN_NAK) == USB_EP_INT_OUT_TRAN_NAK &&
					   (g_EP0InterruptFactor & 0x33) == 0x00 &&
					   (UsbGetMainInt() & USB_MAIN_INT_RCVSETUP) == 0){

						EP0Retry = 0;

						// change OUT direction
						UsbSetEP0Direction(USB_EP_OUT);

						// Valid RequestLength & Transfer complete
						if(RequestLength != 0 && g_EP0ActualLength == 0){

							// Clear ForceNAK (receive 0 length packet)
							UsbSetEP0Control(USB_EP0_OUT, 0, USB_EP_CTL_NAK);

							// change Receive get state
							g_EP0State = EP0_DATA_GET;

						} else {
							// Nothing receive data or Abort receive
							// ForceSTALL
							UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_STALL, 0);
							UsbSetEP0Control(USB_EP0_OUT, USB_EP_CTL_STALL, 0);

							g_EP0State = EP0_IDLE;
						}
					}
				} else {
					// OUT direction

					// Valid IN_TranNAK
					if((UsbGetEP0Int() & USB_EP_INT_IN_TRAN_NAK) == USB_EP_INT_IN_TRAN_NAK &&
					   (g_EP0InterruptFactor & 0x33) == 0x00 &&
					   (UsbGetMainInt() & USB_MAIN_INT_RCVSETUP) == 0){

						EP0Retry = 0;

						// change IN direction
						UsbSetEP0Direction(USB_EP_IN);

						// Valid g_EP0ActualLength
						if(g_EP0ActualLength){
							// ForceSTALL
							UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_STALL, 0);
							UsbSetEP0Control(USB_EP0_OUT, USB_EP_CTL_STALL, 0);

							g_EP0State = EP0_IDLE;
						} else {
							// Send 0 length packet
							UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_ENSHORTPKT, USB_EP_CTL_NAK);

							g_EP0State = EP0_DATA_SEND;
						}
					}
				}
			}
		}
	}
}

/*************************************************************
	Idle()

	description:
		None process.

	argument	: None
	return		: None

**************************************************************/
static void Idle(void){

}
