/***************************************************************************
*
*	File name : usb_drv.c
*	Function  : USB driver (C17801)
*	Revision  :
*		August. 20, 2007	Yakushiji		First revision
*
****************************************************************************/

// Include File
#include	"reg_801.h"
#include	"system.h"
#include	"usb_api.h"
#include	"usb_usb.h"
#include	"descriptor.h"
#include	"usb_request.h"
#include	"usb_control.h"
#include	"usb_bulk.h"


T_USB_CFG						g_Config;	// save initialize config data

// Local interrupt factor
static volatile unsigned char	MainInterruptFactor;
static volatile unsigned char	SIEInterruptFactor;
static volatile unsigned char	EPrInterruptFactor;
static volatile unsigned char	FIFOInterruptFactor;
static volatile unsigned char	EP0InterruptFactor;
#ifdef	FOR_BULK_TRANSFER
static volatile unsigned char	EPaInterruptFactor;
static volatile unsigned char	EPbInterruptFactor;
static volatile unsigned char	EPcInterruptFactor;
static volatile unsigned char	EPdInterruptFactor;
#endif

// Save interrupt factor
volatile unsigned char		g_MainInterruptFactor;
volatile unsigned char		g_SIEInterruptFactor;
volatile unsigned char		g_EPrInterruptFactor;
volatile unsigned char		g_FIFOInterruptFactor;
volatile unsigned char		g_EP0InterruptFactor;
#ifdef	FOR_BULK_TRANSFER
volatile unsigned char		g_EPaInterruptFactor;
volatile unsigned char		g_EPbInterruptFactor;
volatile unsigned char		g_EPcInterruptFactor;
volatile unsigned char		g_EPdInterruptFactor;
#endif

unsigned short				g_UsbFifoMap[USB_INTERFACE_NUM][6];			// FIFO area map

unsigned short				g_DeviceDescPoint;							// Device descriptor pointer
unsigned short				g_ConfigDescPoint[USB_CONFIG_NUM];			// Config descriptor pointer
unsigned short				g_InterfaceDescPoint[USB_INTERFACE_NUM];	// Interface descriptor pointer
unsigned short				g_EndpointDescPoint;						// Endpoint descriptor pointer
unsigned short				g_StringDescPoint;							// String descriptor pointer

unsigned char				g_RemoteWakeupSupport;						// RemoteWakeup support flag
unsigned short				g_MaxPacket[USB_INTERFACE_NUM][5];			// MaxPacket
unsigned char				g_EndpointNum[USB_INTERFACE_NUM][4];		// Endpoint number
unsigned char				g_Attributes[USB_INTERFACE_NUM][4];			// Endpoint attribute
unsigned short				g_DescTotalLength;							// Descriptor total length

volatile T_MANAGEMENT_FLAG	g_ManagementFlag;							// Management flag

/*************************************************************
	UsbInit()

	description:
		Initialize of USB controller.

	argument	:
		config	: USB controller configuration
	return		:
		SUCCESS	: Normal Complete
		ERROR	: Error Complete

**************************************************************/
int UsbInit(T_USB_CFG *config){

	int				ret = ERROR;
	unsigned char	*p;
	int				i,j,k;
	short			temp;
	unsigned short	remain;
	unsigned char	flag;
	int				configuration;
	int				interface;
	unsigned char	type;
	unsigned char	size;

	// Clear Descriptor information
	for(i = 0; i < USB_INTERFACE_NUM; i++){
		for(j = 0; j < 6; j++){
			g_MaxPacket[i][j] = 0;
			if(j < 4){
				g_EndpointNum[i][j] = g_Attributes[i][j] = 0;
			}
		}
	}

	// Clear descriptor pointer
	g_DeviceDescPoint		= 0;
	for(i = 0; i < USB_CONFIG_NUM; i++){
		g_ConfigDescPoint[i] = 0;
	}
	for(i = 0; i < USB_INTERFACE_NUM; i++){
		g_InterfaceDescPoint[i] = 0;
	}
	g_EndpointDescPoint	= 0;
	g_StringDescPoint	= 0;

	// Descriptor total length
	g_DescTotalLength = 0;

	// clear Remote wakeup support
	g_RemoteWakeupSupport	= 0;

	// save Descriptor data pointer
	g_Config.descriptorData = config->descriptorData;
	// save Descriptor size
	g_Config.descriptorSize = config->descriptorSize;

#ifdef	FOR_NEGOTIATION
	// save Auto negotiation mode
	g_Config.negotiationMode = config->negotiationMode;
#endif

#ifdef	FOR_ADDRESS
	// save Auto address reply mode
	g_Config.addressMode = config->addressMode;
#endif

#ifdef	FOR_MANAGEMENT
	// save BUS processing mode of USB
	g_Config.managementMode = config->managementMode;
#endif

	// save Active clock function
	if((g_Config.activeFunc = config->activeFunc) == NULL){
		return ret;
	}
#ifndef	FOR_SNOOZE_OFF
	// save Snooze clock function
	if((g_Config.snoozeFunc = config->snoozeFunc) == NULL){
		return ret;
	}
#endif
#ifdef	FOR_CLASS_REQUEST
	// save Reset process function
	if((g_Config.resetFunc = config->resetFunc) == NULL){
		return ret;
	}
#endif
#ifdef	FOR_REMOTEWAKEUP
	// save RemoteWakeup process function
	if((g_Config.remoteFunc = config->remoteFunc) == NULL){
		return ret;
	}
#endif
#ifdef	FOR_CLASS_REQUEST
	// save Class request function
	if((g_Config.classFunc = config->classFunc) == NULL){
		return ret;
	}
#endif
	// save Vendor request function
	if((g_Config.vendorFunc = config->vendorFunc) == NULL){
		return ret;
	}
	// save USB interrupt enable function
	if((g_Config.intEnableFunc = config->intEnableFunc) == NULL){
		return ret;
	}
	// save USB interrupt mask function
	if((g_Config.intDisableFunc = config->intDisableFunc) == NULL){
		return ret;
	}

	// Serch Endpoint/Attribute/MaxPacket
	p = g_Config.descriptorData;
	i = j = 0;
	configuration = 0;
	interface = 0;

	while(1){
		// if descriptor pointer over then break
		if(p >= (g_Config.descriptorData + g_Config.descriptorSize)){
			break;
		}

		size = *p;			// descriptor size
		type = *(p + 1);	// descriptor type

		// find Device Descriptor
		if(type == DEVICE_DESCTYPE){

			// Set descriptor pointer
			if(g_DeviceDescPoint == 0){
				g_DeviceDescPoint = (unsigned short)(p - g_Config.descriptorData);
			}

			g_MaxPacket[0][0] = *(unsigned char *)(p + 7);

		// find Config Descriptor
		} else if(type == CONFIGURATION_DESCTYPE){


			((unsigned char *)&temp)[0] = *(unsigned char *)(p + 2);
			((unsigned char *)&temp)[1] = *(unsigned char *)(p + 3);

			g_DescTotalLength += temp;

			// set Remote wakeup support
			g_RemoteWakeupSupport |= (*(p + 7) & 0x20);

			// Set descriptor pointer
			if(g_ConfigDescPoint[configuration] == 0){
				g_ConfigDescPoint[configuration] = (unsigned short)(p - g_Config.descriptorData);
				configuration++;
			}

		// find Interface Descriptor
		} else if(type == INTERFACE_DECTYPE){
			// Set descriptor pointer
			if(g_InterfaceDescPoint[interface] == 0){
				g_InterfaceDescPoint[interface] = (unsigned short)(p - g_Config.descriptorData);
				interface++;
#ifdef	FOR_BULK_TRANSFER
				i = 1;
				j = 0;
#endif
			}
#ifdef	FOR_BULK_TRANSFER
		// find Endpoint Descriptor
		} else if(type == ENDPOINT_DECTYPE){

			// Set descriptor pointer
			if(g_EndpointDescPoint == 0){
				g_EndpointDescPoint = (unsigned short)(p - g_Config.descriptorData);
			}

			g_EndpointNum[interface - 1][j]  = *(p + 2);
			g_Attributes[interface - 1][j++] = *(p + 3);
			((unsigned char *)&g_MaxPacket[interface - 1][i])[0]   = *(unsigned char *)(p + 4);
			((unsigned char *)&g_MaxPacket[interface - 1][i++])[1] = *(unsigned char *)(p + 5);
#endif

		// find String Descriptor
		} else if(type == STRING_DESCTYPE){
			// Set descriptor pointer
			if(g_StringDescPoint == 0){
				g_StringDescPoint = (unsigned short)(p - g_Config.descriptorData);
			}
		}

		p += size;

		// if support endpoint over then break
		if(i >= 5 && interface >= USB_INTERFACE_NUM){
			break;
		}
	}

#ifdef	FOR_BULK_TRANSFER
	// set & calculate FIFO allocate
	for(i = 0; i < USB_INTERFACE_NUM; i++){
		g_UsbFifoMap[i][0] = g_MaxPacket[0][0];
		g_UsbFifoMap[i][1] = (g_Config.descriptorSize + 3 + g_MaxPacket[0][0]) & 0xFFFC;

		temp = remain = 1024 - g_UsbFifoMap[i][1];

		flag = 0;
		k = 1;

		while(1){
			for(j = 1; j <= 4 ; j++){
				if(g_MaxPacket[i][j] != 0){

					temp -= (g_MaxPacket[i][j] * k);

					g_UsbFifoMap[i][j + 1] = g_UsbFifoMap[i][j] + (g_MaxPacket[i][j] * k);
				} else {
					g_UsbFifoMap[i][j + 1] = 0;
				}
			}

			if(g_MaxPacket[i][1] == 0 &&
			   g_MaxPacket[i][2] == 0 &&
			   g_MaxPacket[i][3] == 0 &&
			   g_MaxPacket[i][4] == 0){
				break;
			}

			if(flag){
				break;
			}

			if(temp > 0){
				temp = remain;
				k++;
			} else if(temp < 0){
				temp = remain;
				k--;
				flag++;
			} else {
				break;
			}
		}
	}
#else
	for(i = 0; i < USB_INTERFACE_NUM; i++){
		g_UsbFifoMap[i][0] = g_MaxPacket[0][0];
		g_UsbFifoMap[i][1] = (g_Config.descriptorSize + 3 + g_MaxPacket[0][0]) & 0xFFFC;
	}
#endif

	g_UsbSnoozeFlag = 1;

	// Active
	ActiveUSBIP();

	// Set FIFO area
	UsbSetFIFOArea(&g_UsbFifoMap[0][0]);

	// Write Descriptor Data to FIFO
	UsbWriteDescriptor(g_Config.descriptorData, 0, g_Config.descriptorSize);

	// Clear All Interrupt factor
	UsbClearMainInt(0xFF);
	UsbClearSIEInt(0xFF);
	UsbClearEPrInt(0xFF);
	UsbClearFIFOInt(0xFF);
	UsbClearEP0Int(0xFF);
#ifdef	FOR_BULK_TRANSFER
	UsbClearEPaInt(0xFF);
	UsbClearEPbInt(0xFF);
	UsbClearEPcInt(0xFF);
	UsbClearEPdInt(0xFF);
#endif

	// Disable All interrupt
	UsbMainIntDisable(0xFF);
	UsbSIEIntDisable(0xFF);
	UsbEPrIntDisable(0xFF);
	UsbFIFOIntDisable(0xFF);

#ifdef	FOR_BULK_TRANSFER
	// Enable MainInt
	UsbMainIntEnable(USB_MAIN_INT_EP0 | USB_MAIN_INT_FIFO | USB_MAIN_INT_EPR | USB_MAIN_INT_SIE);
#else
	// Enable MainInt
	UsbMainIntEnable(USB_MAIN_INT_EP0 | USB_MAIN_INT_FIFO | USB_MAIN_INT_SIE);
#endif

	// Enable SIEInt
	UsbSIEIntEnable(USB_SIE_INT_VBUS | USB_SIE_INT_SETADDRESSCMP);

	UsbFIFOIntEnable(USB_FIFO_INT_DESCCMP);

	// Set Max Packet size for Control pipe
	UsbSetEndpointMaxSize(USB_ENDPOINT_0, 0x40);


	// Set USB Configuration
	USB_USBControl	= 0x00;
	USB_XcvrControl	= 0x01;	// non-drive
	USB_USBTest		= 0x00;	// Clear test mode

	g_UsbState = USB_DEFAULT_STATE;			// USB Device state
	g_EP0State = EP0_IDLE;					// Endpoint0 state
	g_EP0Result = EP0_NO_RESULT;			// Endpoint0 result

#ifdef	FOR_BULK_TRANSFER
	for(i = 0; i < 4; i++){
		g_EPxState[i]  = USB_EPX_IDLE;		// Endpoint x state
		g_EPxResult[i] = USB_EPX_NO_RESULT;	// Endpoint x result
	}
#endif

	g_ManagementFlag.Init				= 1;
	g_ManagementFlag.USBReqRcvd			= 0;
	g_ManagementFlag.EnbRemoteWakeup	= 0;
#ifdef	FOR_TEST_MODE
	g_ManagementFlag.TestMode			= 0;
#endif
#ifdef	FOR_ADDRESS
	g_ManagementFlag.SetAddress			= 0;
#endif
	g_ManagementFlag.DetachProg			= 1;
	g_ManagementFlag.Attach				= 1;
	g_ManagementFlag.Detach				= 0;
	g_ManagementFlag.Reset				= 0;
	g_ManagementFlag.Suspend			= 0;
	g_ManagementFlag.Resume				= 0;
	g_ManagementFlag.NonJ				= 0;
	g_ManagementFlag.VbusChange			= 0;

	return SUCCESS;
}

/*************************************************************
	UsbBusManagement()

	description:
		BUS management processing.

	argument	: None
	return		: None

**************************************************************/
void UsbBusManagement(void){

#ifdef	FOR_MANAGEMENT
	if(g_Config.managementMode == USB_MANAGEMENT_AUTO){
#endif

		if(g_ManagementFlag.Attach){
			if(UsbAttach() == ERROR){
				return;
			}
		}

		if(g_ManagementFlag.Detach){
			UsbDetach();
			return;
		}

		if(g_ManagementFlag.Reset){
			UsbReset();
			return;
		}

		if(g_ManagementFlag.Suspend){
			UsbSuspend();
		}

		if(g_ManagementFlag.Resume){
			UsbResume();
			return;
		}

		if(g_ManagementFlag.USBReqRcvd){
			if(g_EP0State == EP0_IDLE){
				g_EP0State = EP0_REQ_CHECK;
			}
		}

		UsbCtrlProcess();

#ifdef	FOR_MANAGEMENT
	} else {
		if(g_ManagementFlag.USBReqRcvd){
			if(g_EP0State == EP0_IDLE){
				g_EP0State = EP0_REQ_CHECK;
			}
		}
	}
#endif
}

/*************************************************************
	UsbInterruptHandler()

	description:
		USB-IP interrupt handler processing.

	argument	: None
	return		: None

**************************************************************/
void UsbInterruptHandler(void){

	unsigned char	clear;

	MainInterruptFactor = UsbGetMainInt();
	g_MainInterruptFactor |= MainInterruptFactor;

#if 1
	clear = 0;

	// Valid SIE interrupt
	if(g_MainInterruptFactor & USB_MainIntStat_SIEIntStat){

		SIEInterruptFactor = UsbGetSIEInt();

		// Valid VBUS changed
		if(SIEInterruptFactor & USB_SIEIntStat_VBUSChanged){
			// request Vbus Change
			g_ManagementFlag.VbusChange = 1;

			clear |= USB_SIEIntStat_VBUSChanged;
		}

#ifdef	FOR_NEGOTIATION
		if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){
			if(SIEInterruptFactor & USB_SIEIntStat_DetectJ){
				// Disable Detect-J
				UsbSIEIntDisable(USB_SIE_INT_DETECTJ);
				// J-State end & Detect Reset/Suspend
				USB_USBControl &= ~USB_USBControl_StartDetectJ;
				USB_USBControl &= ~USB_USBControl_DisBusDetect;

				clear |= USB_SIEIntStat_DetectJ;
			}
		}
#endif
		if(SIEInterruptFactor & USB_SIEIntStat_NonJ){
			// Disable Non-J
			UsbSIEIntDisable(USB_SIE_INT_NONJ);

			// request NonJ
			g_ManagementFlag.NonJ = 1;

			clear |= USB_SIEIntStat_NonJ;
		}

		if(SIEInterruptFactor & USB_SIEIntStat_DetectReset){
			clear |= USB_SIEIntStat_DetectReset;

#ifdef	FOR_NEGOTIATION
			if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){
				// Disable Reset/Suspend
				USB_USBControl |= USB_USBControl_DisBusDetect;
			} else {
				// request reset
				g_ManagementFlag.Reset = 1;
			}
#else
			// request reset
			g_ManagementFlag.Reset = 1;
#endif
		}

		// Valid Suspend
		if(SIEInterruptFactor & USB_SIEIntStat_DetectSuspend){
			// request suspend
			g_ManagementFlag.Suspend = 1;

			clear |= USB_SIEIntStat_DetectSuspend;
		}

		g_SIEInterruptFactor |= (SIEInterruptFactor & ~clear);
		UsbClearSIEInt(SIEInterruptFactor);
	}

#else
	// Valid SIE Interrupt
	if(MainInterruptFactor & USB_MainIntStat_SIEIntStat){

		SIEInterruptFactor = UsbGetSIEInt();

		if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){
			if(SIEInterruptFactor & USB_SIEIntStat_DetectJ){
				// Disable Detect-J
				UsbSIEIntDisable(USB_SIE_INT_DETECTJ);
				// J-State end & Detect Reset/Suspend
				USB_USBControl &= ~(USB_USBControl_StartDetectJ | USB_USBControl_DisBusDetect);
			}
		}
		if(SIEInterruptFactor & USB_SIEIntStat_NonJ){
			// Disable Non-J
			UsbSIEIntDisable(USB_SIE_INT_NONJ);
		}

		if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){
			if(SIEInterruptFactor & USB_SIEIntStat_DetectReset){
				// Disable Reset/Suspend
				USB_USBControl |= USB_USBControl_DisBusDetect;
			}
		}

		g_SIEInterruptFactor |= SIEInterruptFactor;
		UsbClearSIEInt(SIEInterruptFactor);
	}
#endif

#ifdef	FOR_BULK_TRANSFER
	// Valid EPr interrupt
	if(MainInterruptFactor & USB_MainIntStat_EPrIntStat){

		EPrInterruptFactor = UsbGetEPrInt();
		UsbClearEPrInt(EPrInterruptFactor);

		g_EPrInterruptFactor |= EPrInterruptFactor;

		// Valid EPa interrupt
		if(EPrInterruptFactor & USB_EPrIntStat_EPaIntStat){

			EPaInterruptFactor = UsbGetEPaInt();
			UsbClearEPaInt(EPaInterruptFactor);

			g_EPaInterruptFactor |= EPaInterruptFactor;
		}

		// Valid EPb interrupt
		if(EPrInterruptFactor & USB_EPrIntStat_EPbIntStat){

			EPbInterruptFactor = UsbGetEPbInt();
			UsbClearEPbInt(EPbInterruptFactor);

			g_EPbInterruptFactor |= EPbInterruptFactor;
		}

		// Valid EPc interrupt
		if(EPrInterruptFactor & USB_EPrIntStat_EPcIntStat){

			EPcInterruptFactor = UsbGetEPcInt();
			UsbClearEPcInt(EPcInterruptFactor);

			g_EPcInterruptFactor |= EPcInterruptFactor;
		}

		// Valid EPd interrupt
		if(EPrInterruptFactor & USB_EPrIntStat_EPdIntStat){

			EPdInterruptFactor = UsbGetEPdInt();
			UsbClearEPdInt(EPdInterruptFactor);

			g_EPdInterruptFactor |= EPdInterruptFactor;
		}
	}
#endif

	// Valid FIFO interrupt
	if(MainInterruptFactor & USB_MainIntStat_FIFOIntStat){
		FIFOInterruptFactor = UsbGetFIFOInt();
		UsbClearFIFOInt(FIFOInterruptFactor);

		g_FIFOInterruptFactor |= FIFOInterruptFactor;
	}

	// Valid EP0 interrupt
	if(MainInterruptFactor & USB_MainIntStat_EP0IntStat){

		EP0InterruptFactor = UsbGetEP0Int();
		UsbClearEP0Int(EP0InterruptFactor);

		g_EP0InterruptFactor |= EP0InterruptFactor;
	}

	// Clear MainInt
	UsbClearMainInt(MainInterruptFactor & 0xFE);
}

/*************************************************************
	UsbCheckInterrupt()

	description:
		Check interrupt factor.

	argument	: None
	return		: None

**************************************************************/
void UsbCheckInterrupt(void){

	InterruptDisableUSB();

#if 1
	if(g_ManagementFlag.VbusChange == 1){
		// check VBUS
		if(UsbVBUSCheck() == USB_VBUS_OFF){
			// request detach
			g_ManagementFlag.Detach = 1;
			g_ManagementFlag.Attach = 0;
		} else {
			g_ManagementFlag.Attach = 1;
			g_ManagementFlag.Detach = 0;
		}

		g_ManagementFlag.Reset = 0;
		g_ManagementFlag.Suspend = 0;
		g_ManagementFlag.NonJ = 0;
		g_ManagementFlag.VbusChange = 0;
	}

	// Valid SIE interrupt
	if(g_MainInterruptFactor & USB_MainIntStat_SIEIntStat){

#else
	// Valid SIE interrupt
	if(g_MainInterruptFactor & USB_MainIntStat_SIEIntStat){

		// Valid VBUS changed
		if(g_SIEInterruptFactor & USB_SIEIntStat_VBUSChanged){
			// check VBUS
			if(UsbVBUSCheck() == USB_VBUS_OFF){
				// request detach
				g_ManagementFlag.Detach = 1;
				g_ManagementFlag.Attach = 0;
				g_ManagementFlag.Reset = 0;
				g_ManagementFlag.Suspend = 0;
				g_ManagementFlag.NonJ = 0;

				// clear factor
				g_MainInterruptFactor = 0;
			} else {
				g_ManagementFlag.Attach = 1;
				g_ManagementFlag.Detach = 0;
				g_ManagementFlag.Reset = 0;
				g_ManagementFlag.Suspend = 0;
				g_ManagementFlag.NonJ = 0;

				// clear factor
				g_MainInterruptFactor = 0;
			}
		} else {
			// Valid NonJ
			if(g_SIEInterruptFactor & USB_SIEIntStat_NonJ){
				// request NonJ
				g_ManagementFlag.NonJ = 1;

				// clear factor
				g_MainInterruptFactor = 0;
			}
			// Valid reset
			if(g_SIEInterruptFactor & USB_SIEIntStat_DetectReset){

				// request reset
				g_ManagementFlag.Reset = 1;

				// clear factor
				g_MainInterruptFactor = 0;
			} else {
				// Valid Suspend
				if(g_SIEInterruptFactor & USB_SIEIntStat_DetectSuspend){

					// request suspend
					g_ManagementFlag.Suspend = 1;

					// clear factor
					g_MainInterruptFactor = 0;
				}
			}
		}
#endif

		// Valid Address complete
		if(g_SIEInterruptFactor & USB_SIEIntStat_SetAddressCmp){

			// Address is 0
			if((USB_USBAddress & USB_USBAddress_USBAddress) == 0x00){
				// change Default state
				g_UsbState = USB_DEFAULT_STATE;
			} else {
				// change Address state
				g_UsbState = USB_ADDRESS_STATE;
			}
		}

		// clear factor
		g_SIEInterruptFactor = 0;
	}

	// Valid FIFO interrupt
	if(g_MainInterruptFactor & USB_MainIntStat_FIFOIntStat){
		// Valid descriptor complete
		if(g_FIFOInterruptFactor & USB_FIFOIntStat_DescriptorCmp){
			g_EP0Result = DESCRIPTOR_CMP;
		}
		// clear factor
		g_FIFOInterruptFactor = 0;
	}

	// Valid EP0 interrupt
	if(g_MainInterruptFactor & USB_MainIntStat_EP0IntStat){

		// interrupt mask
		g_EP0InterruptFactor &= USB_EP0IntEnb;

		// Valid IN transaction error
		if(g_EP0InterruptFactor & USB_EP0IntStat_INTranErr){
			g_EP0Result = EP0_IN_ERR;
		}

		// Valid OUT transaction error
		if(g_EP0InterruptFactor & USB_EP0IntStat_OUTTranErr){
			g_EP0Result = EP0_OUT_ERR;
		}

		// Valid IN transaction ACK
		if(g_EP0InterruptFactor & USB_EP0IntStat_INTranACK){
			g_EP0Result = EP0_IN_ACK;
		}

		// Valid OUT transaction ACK
		if(g_EP0InterruptFactor & USB_EP0IntStat_OUTTranACK){
			g_EP0Result = EP0_OUT_ACK;
		}

		// clear factor
		g_EP0InterruptFactor = 0;
	}

#ifdef	FOR_BULK_TRANSFER
	// Valid EPr interrupt
	if(g_MainInterruptFactor & USB_MainIntStat_EPrIntStat){
		// Valid EPa interrupt
		if(g_EPrInterruptFactor & USB_EPrIntStat_EPaIntStat){

			// interrupt mask
			g_EPaInterruptFactor &= USB_EPaIntEnb;

			// Valid IN transaction error
			if(g_EPaInterruptFactor & USB_EPaIntStat_INTranErr){
				g_EPxResult[0] = USB_EPX_IN_ERR;
			}

			// Valid OUT transaction error
			if(g_EPaInterruptFactor & USB_EPaIntStat_OUTTranErr){
				g_EPxResult[0] = USB_EPX_OUT_ERR;
			}

			// Valid IN transaction ACK
			if(g_EPaInterruptFactor & USB_EPaIntStat_INTranACK){
				g_EPxResult[0] = USB_EPX_IN_ACK;
			}

			// Valid OUT transaction ACK
			if(g_EPaInterruptFactor & USB_EPaIntStat_OUTTranACK){
				g_EPxResult[0] = USB_EPX_OUT_ACK;
			}

			// clear factor
			g_EPaInterruptFactor = 0;
		}

		// Valid EPb interrupt
		if(g_EPrInterruptFactor & USB_EPrIntStat_EPbIntStat){

			// interrupt mask
			g_EPbInterruptFactor &= USB_EPbIntEnb;

			// Valid IN transaction error
			if(g_EPbInterruptFactor & USB_EPbIntStat_INTranErr){
				g_EPxResult[1] = USB_EPX_IN_ERR;
			}

			// Valid OUT transaction error
			if(g_EPbInterruptFactor & USB_EPbIntStat_OUTTranErr){
				g_EPxResult[1] = USB_EPX_OUT_ERR;
			}

			// Valid IN transaction ACK
			if(g_EPbInterruptFactor & USB_EPbIntStat_INTranACK){
				g_EPxResult[1] = USB_EPX_IN_ACK;
			}

			// Valid OUT transaction ACK
			if(g_EPbInterruptFactor & USB_EPbIntStat_OUTTranACK){
				g_EPxResult[1] = USB_EPX_OUT_ACK;
			}

			// clear factor
			g_EPbInterruptFactor = 0;
		}

		// Valid EPc interrupt
		if(g_EPrInterruptFactor & USB_EPrIntStat_EPcIntStat){

			// interrupt mask
			g_EPcInterruptFactor &= USB_EPcIntEnb;

			// Valid IN transaction error
			if(g_EPcInterruptFactor & USB_EPcIntStat_INTranErr){
				g_EPxResult[2] = USB_EPX_IN_ERR;
			}

			// Valid OUT transaction error
			if(g_EPcInterruptFactor & USB_EPcIntStat_OUTTranErr){
				g_EPxResult[2] = USB_EPX_OUT_ERR;
			}

			// Valid IN transaction ACK
			if(g_EPcInterruptFactor & USB_EPcIntStat_INTranACK){
				g_EPxResult[2] = USB_EPX_IN_ACK;
			}

			// Valid OUT transaction ACK
			if(g_EPcInterruptFactor & USB_EPcIntStat_OUTTranACK){
				g_EPxResult[2] = USB_EPX_OUT_ACK;
			}

			// clear factor
			g_EPcInterruptFactor = 0;
		}

		// Valid EPd interrupt
		if(g_EPrInterruptFactor & USB_EPrIntStat_EPdIntStat){

			// interrupt mask
			g_EPdInterruptFactor &= USB_EPdIntEnb;

			// Valid IN transaction error
			if(g_EPdInterruptFactor & USB_EPdIntStat_INTranErr){
				g_EPxResult[3] = USB_EPX_IN_ERR;
			}

			// Valid OUT transaction error
			if(g_EPdInterruptFactor & USB_EPdIntStat_OUTTranErr){
				g_EPxResult[3] = USB_EPX_OUT_ERR;
			}

			// Valid IN transaction ACK
			if(g_EPdInterruptFactor & USB_EPdIntStat_INTranACK){
				g_EPxResult[3] = USB_EPX_IN_ACK;
			}

			// Valid OUT transaction ACK
			if(g_EPdInterruptFactor & USB_EPdIntStat_OUTTranACK){
				g_EPxResult[3] = USB_EPX_OUT_ACK;
			}

			// clear factor
			g_EPcInterruptFactor = 0;
		}

		// clear factor
		g_EPrInterruptFactor = 0;
	}
#endif

	// Receive Setup packet
	if(UsbGetMainInt() & USB_MAIN_INT_RCVSETUP){
		g_ManagementFlag.USBReqRcvd = 1;
	}

	g_MainInterruptFactor = 0;

	InterruptEnableUSB();
}

