/***************************************************************************
*
*	File name : usb_request.c
*	Function  : USB driver (C17801)
*			Standard request process.
*	Revision  :
*		August. 20, 2007	Yakushiji		First revision
*
****************************************************************************/

#include	"reg_801.h"
#include	"system.h"
#include	"usb_api.h"
#include	"descriptor.h"
#include	"usb_request.h"
#include	"usb_control.h"
#include	"usb_bulk.h"

#define	RECIPIENT_DEVICE		(0x00)
#define	RECIPIENT_INTERFACE		(0x01)
#define	RECIPIENT_ENDPOINT		(0x02)

#define	DEVICE_REMOTE_WAKEUP	(0x01)
#define	TEST_MODE				(0x02)
#define	ENDPOINT_HALT			(0x00)

#define	TEST_J_MODE				(0x01)
#define	TEST_K_MODE				(0x02)
#define	TEST_SE0_NAK_MODE		(0x03)
#define	TEST_PACKET_MODE		(0x04)

unsigned char			g_UsbState;				// USB Device state
unsigned char			g_CurrentAlterSet;		// Current alternate interface

unsigned short			g_EP0TotalLength;		// Endpoint0 total length
unsigned short			g_EP0ActualLength;		// Endpoint0 actual length

static unsigned char	EndpointCount;

// Prototype
int			StandardRequest(void);
static int	GetStatus(void);
#ifdef	FOR_CLEAR_FEATURE
static int	ClearFeature(void);
#endif
#ifdef	FOR_SET_FEATURE
static int	SetFeature(void);
#endif
static int	SetAddress(void);
static int	GetDescriptor(void);
static int	GetConfiguration(void);
static int	SetConfiguration(void);
#ifdef	FOR_GET_INTERFACE
static int	GetInterface(void);
#endif
#ifdef	FOR_SET_INTERFACE
static int	SetInterface(void);
#endif
static void	SetAlternateInterface(unsigned char);
static int	RequestError(void);

#ifdef	FOR_TEST_MODE
static const unsigned char	TestPacket[] = {
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,
	0xAA,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,
	0xEE,0xFE,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0x7F,0xBF,0xDF,
	0xEF,0xF7,0xFB,0xFD,0xFC,0x7E,0xBF,0xDF,
	0xEF,0xF7,0xFB,0xFD,0x7E
};
#endif


// Standard request table
static int (*StandardRequestTable[])(void) = {
	GetStatus,				// 00h Get Status
#ifdef	FOR_CLEAR_FEATURE
	ClearFeature,			// 01h Clear Feature
#else
	RequestError,			// 01h Clear Feature
#endif
	RequestError,			// 02h Reserved for future use
#ifdef	FOR_SET_FEATURE
	SetFeature,				// 03h Set Feature
#else
	RequestError,			// 03h Set Feature
#endif
	RequestError,			// 04h Reserved for future use
	SetAddress,				// 05h Set Address
	GetDescriptor,			// 06h Get Descriptor
	RequestError,			// 07h Set Descriptor
	GetConfiguration,		// 08h Get Configuration
	SetConfiguration,		// 09h Set Configuration
#ifdef	FOR_GET_INTERFACE
	GetInterface,			// 0Ah Get Interface
#else
	RequestError,			// 0Ah Get Interface
#endif
#ifdef	FOR_SET_INTERFACE
	SetInterface,			// 0Bh Set Interface
#else
	RequestError,			// 0Bh Set Interface
#endif
	RequestError			// 0Ch Synch Frame
};

/*************************************************************
	StandardRequest()

	description:
		Check Standard request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
int StandardRequest(void)
{
	int		ret = ERROR;

	if(g_RequestPacket.request < 0x0D){
		ret = StandardRequestTable[g_RequestPacket.request]();
	}

	return ret;
}


/*************************************************************
	GetStatus()

	description:
		Get Status request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int GetStatus(void)
{
	int				i, j;
	int				ret = ERROR;
	char			buffer[2];
	unsigned short	length;

	// check state
	if(g_UsbState != USB_DEFAULT_STATE){

		// check direction
		if((g_RequestPacket.requestType & 0x80) == 0x80){

			switch(g_RequestPacket.requestType & 0x1F){
			case RECIPIENT_DEVICE:
				// check parameter
				if(g_RequestPacket.value){
					break;
				}
				if(g_RequestPacket.index){
					break;
				}
				if(g_RequestPacket.length != 0x02){
					break;
				}

				// set reply data
				buffer[0] = (g_ManagementFlag.EnbRemoteWakeup << 1) + 0x01;	// Self Powerd(0x01)
				buffer[1] = 0x00;

				ret = SUCCESS;
				break;

			case RECIPIENT_INTERFACE:
				if(g_UsbState == USB_ADDRESS_STATE){
					break;
				}
				// check parameter
				if(g_RequestPacket.value){
					break;
				}
				if(g_RequestPacket.index){
					break;
				}
				if(g_RequestPacket.length != 0x02){
					break;
				}
				// set reply data
				buffer[0] = buffer[1] = 0x00;

				ret = SUCCESS;
				break;

			case RECIPIENT_ENDPOINT:
				if(g_RequestPacket.value){	/* wValue */
					break;
				}
				if(g_RequestPacket.length != 0x02){	/* wLength */
					break;
				}

				// Endpoint 0
				if(g_RequestPacket.index == 0){

					if((UsbGetEP0Control(USB_EP0_IN) & USB_EP_CTL_STALL) ||
					   (UsbGetEP0Control(USB_EP0_OUT) & USB_EP_CTL_STALL)){
						// set reply data
						buffer[0] = 0x01;
					} else {
						// set reply data
						buffer[0] = 0x00;
					}

					buffer[1] = 0x00;

					ret = SUCCESS;

#ifdef	FOR_BULK_TRANSFER
				} else {
					if(g_UsbState == USB_ADDRESS_STATE){
						break;
					}

					for(i = 0, j = 1; i < EndpointCount; i++, j++){
						if(g_RequestPacket.index == (g_EndpointNum[g_CurrentAlterSet][i] & 0x8F)){
							if((UsbGetEndpointControl(j) & USB_EP_CTL_STALL)){
								buffer[0] = 0x01;
							} else {
								buffer[0] = 0x00;
							}

							buffer[1] = 0x00;

							ret = SUCCESS;
						}
					}
#endif
				}
			}
		}
	}

	if(ret == SUCCESS){
		g_EP0TotalLength = g_RequestPacket.length;
		g_EP0ActualLength = 0;

		UsbFIFOWrite(USB_ENDPOINT_0, buffer, g_RequestPacket.length, &length);
		UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_ENSHORTPKT, 0);
		g_EP0State = EP0_DATA_SEND;
		g_EP0Result = EP0_NO_RESULT;

		// ForceNAK off for EP0 control
		UsbSetEP0Control(USB_EP0_IN, 0, USB_EP_CTL_NAK);
	}

	return ret;
}

#ifdef	FOR_CLEAR_FEATURE
/*************************************************************
	ClearFeature()

	description:
		Clear Feature request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int ClearFeature(void)
{
	int			i, j;
	int			ret = ERROR;


	// check state
	if(g_UsbState != USB_DEFAULT_STATE){

		// check direction
		if((g_RequestPacket.requestType & 0x80) == 0x00){

			switch(g_RequestPacket.requestType & 0x1F){
			case RECIPIENT_DEVICE:
				if(g_RequestPacket.value == DEVICE_REMOTE_WAKEUP){
					if(g_RequestPacket.index){	/* wIndex */
						break;
					}
					if(g_RequestPacket.length){	/* wLength */
						break;
					}
					if(g_RemoteWakeupSupport == 0){
						break;
					}
					g_ManagementFlag.EnbRemoteWakeup = 0;	// Disable RemoteWakeup
					ret = SUCCESS;
				} else {
					if(g_RequestPacket.value == TEST_MODE){
						if(g_RequestPacket.index){	/* wIndex */
							break;
						}
						if(g_RequestPacket.length){	/* wLength */
							break;
						}

						ret = SUCCESS;
					}
				}
				break;
			case RECIPIENT_ENDPOINT:
				if(g_RequestPacket.value != ENDPOINT_HALT){
					break;
				}
				if(g_RequestPacket.length){	/* wLength */
					break;
				}

				if(g_RequestPacket.index != 0){

					if(g_UsbState == USB_ADDRESS_STATE){
						break;
					}

#ifdef	FOR_BULK_TRANSFER
					// Search endpoint
					for(i = 0, j = 1; i < EndpointCount; i++, j++){

						// match endpoint number
						if(g_RequestPacket.index == (g_EndpointNum[g_CurrentAlterSet][i] & 0x8F)){

							// clear toggle
							UsbSetEndpointControl(j, USB_EP_CTL_TOGGLE_CLR, 0);

							// check STALL
							if((UsbGetEndpointControl(j) & USB_EP_CTL_STALL)){

								// Clear FIFO
								UsbFIFOClear(j);

								// nomal error
								if((g_EPxState[i] & USB_EPX_RESET) == 0){
									// cancel STALL
									UsbSetEndpointControl(j, 0, USB_EP_CTL_STALL);

									// data trans phase
									if((g_EPxState[i] & 0x0F) <= USB_EPX_DATA_GET){
										g_EPxState[i] = USB_EPX_FEATURE;
									} else {
										g_EPxState[i] = USB_EPX_IDLE;
									}
								}
							}

							ret = SUCCESS;
							break;
						}
					}
#endif
				}
			}
		}
	}

	return ret;
}
#endif

#ifdef	FOR_SET_FEATURE
/*************************************************************
	SetFeature()

	description:
		Set Feature request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int SetFeature(void)
{
	int				i, j;
	int				ret = ERROR;
#ifdef	FOR_TEST_MODE
	unsigned char	testMode;
#endif
	unsigned short	map[5];
	unsigned short	length;

	// check direction
	if((g_RequestPacket.requestType & 0x80) == 0x00){

		switch(g_RequestPacket.requestType & 0x1F){
		case RECIPIENT_DEVICE:
			if(g_RequestPacket.value == DEVICE_REMOTE_WAKEUP){
				if(g_UsbState == USB_DEFAULT_STATE){
					break;
				}
				if(g_RequestPacket.index){	/* wIndex */
					break;
				}
				if(g_RequestPacket.length){	/* wLength */
					break;
				}
				if(g_RemoteWakeupSupport == 0){
					break;
				}
				g_ManagementFlag.EnbRemoteWakeup = 1;
				ret = SUCCESS;
			} else {
#ifdef	FOR_TEST_MODE
				if(g_RequestPacket.value == TEST_MODE){
					if(g_RequestPacket.length){	/* wLength */
						break;
					}

					switch(g_RequestPacket.index){
					case TEST_PACKET_MODE:
						// The preparations to send TestPacket
						UsbSetEndpointMaxSize(USB_ENDPOINT_D, 0x0040);
						UsbSetEndpointDirection(USB_ENDPOINT_D, USB_EP_IN);

						UsbDisableEndpoint(USB_ENDPOINT_A);
						UsbDisableEndpoint(USB_ENDPOINT_B);
						UsbDisableEndpoint(USB_ENDPOINT_C);
						UsbEnableEndpoint(USB_ENDPOINT_D);

						UsbSetEndpointNumber(USB_ENDPOINT_D, 0x0F);

						UsbGetFIFOArea(&map[0]);

						map[1] = 0;
						map[2] = 0;
						map[3] = 0;
						map[4] = 0x03C0;

						UsbSetFIFOArea(&map[0]);

						UsbFIFOAllClear();

						UsbFIFOWrite(USB_ENDPOINT_D, (unsigned char *)TestPacket, 53, &length);

						testMode = USB_TESTMODE_PACKET;

						ret = SUCCESS;
						break;
					case TEST_J_MODE:
						testMode = USB_TESTMODE_J;

						ret = SUCCESS;
						break;
					case TEST_K_MODE:
						testMode = USB_TESTMODE_K;

						ret = SUCCESS;
						break;
					case TEST_SE0_NAK_MODE:
						testMode = USB_TESTMODE_SE0_NAK;

						ret = SUCCESS;
						break;
					default:
						break;
					}

					if(ret == SUCCESS){
						if((ret = UsbSetTestMode(testMode)) == SUCCESS){
							g_ManagementFlag.TestMode = 1;
						}
					}
				} else {
					ret = ERROR;
				}
#else
				ret = ERROR;
#endif
			}
			break;
		case RECIPIENT_ENDPOINT:
			if(g_UsbState == USB_DEFAULT_STATE){
				break;
			}
			if(g_RequestPacket.value != ENDPOINT_HALT){
				break;
			}
			if(g_RequestPacket.length){	/* wLength */
				break;
			}
			if(g_RequestPacket.index != 0){

				if(g_UsbState == USB_ADDRESS_STATE){
					break;
				}

#ifdef	FOR_BULK_TRANSFER
				for(i = 0, j = 1; i < EndpointCount; i++, j++){
					// match endpoint number
					if(g_RequestPacket.index == (g_EndpointNum[g_CurrentAlterSet][i] & 0x8F)){

						// check STALL
						if((UsbGetEndpointControl(j) & USB_EP_CTL_STALL) != USB_EP_CTL_STALL){

							// Set STALL
							UsbSetEndpointControl(j, USB_EP_CTL_STALL, 0);

							g_EPxState[i] = USB_EPX_STALL;
						}

						ret = SUCCESS;
						break;
					}
				}
#endif
			}
			break;
		default:
			ret = ERROR;
		}
	}
	return ret;
}
#endif

/*************************************************************
	SetAddress()

	description:
		Set Address request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int SetAddress(void)
{
	// check state
	if(g_UsbState == USB_CONFIGURED_STATE){
		return ERROR;
	}

	// check direction
	if((g_RequestPacket.requestType & 0x80) != 0x00){
		return ERROR;
	}

	if((g_RequestPacket.requestType & 0x1F) != RECIPIENT_DEVICE){
		return ERROR;
	}

	if(g_RequestPacket.index){	/* wIndex */
		return ERROR;
	}
	if(g_RequestPacket.length){	/* wLength */
		return ERROR;
	}

	// address over 128.
	if(g_RequestPacket.value > 0x7F){
		return ERROR;
	}

#ifdef	FOR_ADDRESS
	g_ManagementFlag.SetAddress = 0;

	if(g_Config.addressMode == USB_ADDRESS_AUTO){
		UsbSetAutoAddress(USB_AUTOADDRESS_ON);
	} else {
		g_ManagementFlag.SetAddress = 1;
	}
#else
	UsbSetAutoAddress(USB_AUTOADDRESS_ON);
#endif

	return SUCCESS;
}

/*************************************************************
	GetDescriptor()

	description:
		Get Descriptor request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int GetDescriptor(void)
{
	int				i;
	unsigned short	offset;
	int				ret = ERROR;

	// check direction
	if((g_RequestPacket.requestType & 0x80) == 0x80){

		if((g_RequestPacket.requestType & 0x1F) == RECIPIENT_DEVICE){

			switch(((unsigned char *)&g_RequestPacket.value)[1]){
			case DEVICE_DESCTYPE:
				if(((unsigned char *)&g_RequestPacket.value)[0]){	/* wValue.l */
					break;
				}
				if(g_RequestPacket.index){	/* wIndex */
					break;
				}
				if(g_RequestPacket.length != 0){

					offset = g_DeviceDescPoint;
					g_EP0TotalLength = *(unsigned char *)((unsigned long)g_Config.descriptorData +
										(unsigned long)g_DeviceDescPoint);

					// It's done within Request Length.
					if(g_RequestPacket.length < g_EP0TotalLength){
						g_EP0TotalLength = g_RequestPacket.length;
					}
					g_EP0ActualLength = g_EP0TotalLength;

					ret = SUCCESS;
				}
				break;
			case CONFIGURATION_DESCTYPE:
				if(((unsigned char *)&g_RequestPacket.value)[0] > USB_CONFIG_NUM){ // wValue
					break;
				}
				if(g_RequestPacket.index){	/* wIndex */
					break;
				}
				if(g_RequestPacket.length != 0){
					offset = g_ConfigDescPoint[((unsigned char *)&g_RequestPacket.value)[0]];

					if(((unsigned char *)&g_RequestPacket.value)[0] == 0){
						g_EP0TotalLength = g_DescTotalLength;
					} else {
						g_EP0TotalLength = *(unsigned short *)((unsigned long)g_Config.descriptorData +
											(unsigned long)g_ConfigDescPoint[((unsigned char *)&g_RequestPacket.value)[1]] + 2);
					}

					// It's done within Request Length.
					if(g_RequestPacket.length < g_EP0TotalLength){
						g_EP0TotalLength = g_RequestPacket.length;
					}

					g_EP0ActualLength = g_EP0TotalLength;

					ret = SUCCESS;
				}
				break;
			case STRING_DESCTYPE:
				if(g_RequestPacket.length != 0){
					for(i = 0; i <= ((unsigned char *)&g_RequestPacket.value)[0]; i++){

						if(i == 0){
							offset = g_StringDescPoint;
						} else {
							offset += *(unsigned char *)((unsigned long)g_Config.descriptorData +
										(unsigned long)offset);
						}

						if(offset > DescriptorSize){
							break;
						}

						g_EP0TotalLength = *(unsigned char *)((unsigned long)g_Config.descriptorData +
												(unsigned long)offset);

						// It's done within Request Length.
						if(g_RequestPacket.length < g_EP0TotalLength){
							g_EP0TotalLength = g_RequestPacket.length;
						}
						g_EP0ActualLength = g_EP0TotalLength;
					}

					if(i == (((unsigned char *)&g_RequestPacket.value)[0] + 1)){
						ret = SUCCESS;
					}
				}
				break;
			case INTERFACE_POWER_DESCTYPE:
			default:
					break;
			}

			if(ret == SUCCESS){
				// Disable IN Tran ACK
				UsbEndpointIntDisable(USB_ENDPOINT_0, USB_EP_INT_IN_TRAN_ACK);

				// Reply descriptor
				UsbReplyDescriptor(offset, g_EP0TotalLength);

				g_EP0State = EP0_DATA_SEND;
				g_EP0Result = EP0_NO_RESULT;
				// ForceNAK off for EP0 control
//				UsbSetEP0Control(USB_EP0_IN, 0, USB_EP_CTL_NAK);
				USB_EP0ControlIN &= ~USB_EP0ControlIN_ForceNAK;
			}
		}
	}

	return ret;
}

/*************************************************************
	GetConfiguration()

	description:
		Get Configuration request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int GetConfiguration(void)
{
	unsigned char	buff[2];
	unsigned short	length;

	// check direction
	if((g_RequestPacket.requestType & 0x80) != 0x80){
		return ERROR;
	}
	if((g_RequestPacket.requestType & 0x1F) != RECIPIENT_DEVICE){
		return ERROR;
	}

	if(g_UsbState == USB_DEFAULT_STATE){
		return ERROR;
	}
	if(g_RequestPacket.value){
		return ERROR;
	}
	if(g_RequestPacket.index){
		return ERROR;
	}
	if(g_RequestPacket.length != 0x01){
		return ERROR;
	}
	if(g_UsbState == USB_CONFIGURED_STATE){
		buff[0] = 0x01;
	}else{
		buff[0] = 0x00;
	}

	g_EP0TotalLength = g_RequestPacket.length;
	g_EP0ActualLength = 0;

	UsbFIFOWrite(USB_ENDPOINT_0, buff, g_RequestPacket.length, &length);
	UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_ENSHORTPKT, 0);
	g_EP0State = EP0_DATA_SEND;
	g_EP0Result = EP0_NO_RESULT;

	// ForceNAK off for EP0 control
	UsbSetEP0Control(USB_EP0_IN, 0, USB_EP_CTL_NAK);

	return SUCCESS;
}

/*************************************************************
	SetConfiguration()

	description:
		Set Configuration request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int SetConfiguration(void)
{

	// check direction
	if((g_RequestPacket.requestType & 0x80) != 0x00){
		return ERROR;
	}

	if((g_RequestPacket.requestType & 0x1F) != RECIPIENT_DEVICE){
		return ERROR;
	}

	if(g_UsbState == USB_DEFAULT_STATE){
		return ERROR;
	}
	if(g_RequestPacket.index){	/* wIndex */
		return ERROR;
	}
	if(g_RequestPacket.length){	/* wLength */
		return ERROR;
	}
	if(g_RequestPacket.value > 0x01){
		return ERROR;
	}

	if(g_RequestPacket.value){
		// change Configured state
		g_UsbState = USB_CONFIGURED_STATE;
		g_CurrentAlterSet = 0;
		SetAlternateInterface(g_CurrentAlterSet);
	} else {
		// change Address state
		g_UsbState = USB_ADDRESS_STATE;

		// Disable Endpoint
		USB_EPaConfig0 &= ~USB_EPaConfig0_EnEndPoint;
		USB_EPbConfig0 &= ~USB_EPbConfig0_EnEndPoint;
		USB_EPcConfig0 &= ~USB_EPcConfig0_EnEndPoint;
		USB_EPdConfig0 &= ~USB_EPdConfig0_EnEndPoint;
	}

	return SUCCESS;
}

#ifdef	FOR_GET_INTERFACE
/*************************************************************
	GetInterface()

	description:
		Get Interface request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int GetInterface(void)
{

	unsigned char	buff[2];
	unsigned short	length;

	// check direction
	if((g_RequestPacket.requestType & 0x80) != 0x80){
		return ERROR;
	}

	if((g_RequestPacket.requestType & 0x1F) != RECIPIENT_INTERFACE){
		return ERROR;
	}

	if(g_UsbState != USB_CONFIGURED_STATE){
		return ERROR;
	}
	if(g_RequestPacket.value){	/* wValue */
		return ERROR;
	}
	if(g_RequestPacket.index){	/* wIndex.h */
		return ERROR;
	}
	if(g_RequestPacket.length != 0x01){	/* wLength */
		return ERROR;
	}

	if(g_RequestPacket.index){	// Interface number
		return ERROR;
	}

	buff[0] = g_CurrentAlterSet;

	g_EP0TotalLength = g_RequestPacket.length;
	g_EP0ActualLength = 0;

	UsbFIFOWrite(USB_ENDPOINT_0, buff, g_RequestPacket.length, &length);
	UsbSetEP0Control(USB_EP0_IN, USB_EP_CTL_ENSHORTPKT, 0);
	g_EP0State = EP0_DATA_SEND;
	g_EP0Result = EP0_NO_RESULT;

	// ForceNAK off for EP0 control
	UsbSetEP0Control(USB_EP0_IN, 0, USB_EP_CTL_NAK);

	return SUCCESS;
}
#endif

#ifdef	FOR_SET_INTERFACE
/*************************************************************
	SetInterface()

	description:
		Set Interface request.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int SetInterface(void)
{
	// check direction
	if((g_RequestPacket.requestType & 0x80) != 0x00){
		return ERROR;
	}

	if((g_RequestPacket.requestType & 0x1F) != RECIPIENT_INTERFACE){
		return ERROR;
	}

	if(g_UsbState != USB_CONFIGURED_STATE){
		return ERROR;
	}
	if(g_RequestPacket.value){	/* wValue.h */
		return ERROR;
	}
	if(g_RequestPacket.length){	/* wLength */
		return ERROR;
	}

	if(g_RequestPacket.index){	// Interface number
		return ERROR;
	}

	if(g_RequestPacket.value > USB_INTERFACE_NUM){
		return ERROR;
	}

	g_CurrentAlterSet = g_RequestPacket.value;
	SetAlternateInterface(g_RequestPacket.value);

	return SUCCESS;
}
#endif

/*************************************************************
	RequestError()

	description:
		Request Error.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error

**************************************************************/
static int RequestError(void)
{
	int ret = ERROR;

	return ret;
}

/*************************************************************
	SetAlternateInterface()

	description:
		Set AlternateI nterface.

	argument	:
		alternate : Alternate interface number
	return		: None

**************************************************************/
static void SetAlternateInterface(unsigned char alternate)
{
	int				i,j;
	unsigned char	disable;
	unsigned char	enable;

	// Set FIFO area
	UsbSetFIFOArea(&g_UsbFifoMap[alternate][0]);

	// Set Max Packet size of endpoint
	UsbSetEndpointMaxSize(USB_ENDPOINT_0, g_MaxPacket[0][0]);
#ifdef	FOR_BULK_TRANSFER
	UsbSetEndpointMaxSize(USB_ENDPOINT_A, g_MaxPacket[alternate][1]);
	UsbSetEndpointMaxSize(USB_ENDPOINT_B, g_MaxPacket[alternate][2]);
	UsbSetEndpointMaxSize(USB_ENDPOINT_C, g_MaxPacket[alternate][3]);
	UsbSetEndpointMaxSize(USB_ENDPOINT_D, g_MaxPacket[alternate][4]);

	// Set Endpoint number
	for(i = 0, j = 1; i < 4; i++, j++){
		UsbSetEndpointNumber(j, (g_EndpointNum[alternate][i] & 0x0F));
		UsbSetEndpointDirection(j, (g_EndpointNum[alternate][i] & 0x80));

		// ISO
		if((g_Attributes[alternate][i] & 0x03) == 0x01){
			UsbSetEndpointMode(j, USB_EP_ISO);

		// Bulk or Interrupt
		} else if((g_Attributes[alternate][i] & 0x03) == 0x02 || (g_Attributes[alternate][i] & 0x03) == 0x03){
			UsbSetEndpointMode(j, USB_EP_BULK_INTR);
		}

	}

	// Clear Interrupt factor
	UsbClearEPaInt(0xFF);
	UsbClearEPbInt(0xFF);
	UsbClearEPcInt(0xFF);
	UsbClearEPdInt(0xFF);

	UsbEPrIntDisable(0xFF);

	EndpointCount = 0;

	// Clear Endpoint control
	for(i = 0, j = 1; i < 4; i++, j++){
		if(g_MaxPacket[alternate][j] != 0){
			UsbSetEndpointControl(j, (USB_EP_CTL_NAK |USB_EP_CTL_TOGGLE_CLR),
									(USB_EP_CTL_STALL | USB_EP_CTL_DISAFNAK | USB_EP_CTL_ENSHORTPKT |
									 USB_EP_CTL_AUTOFORCENAK));

			// Out direction
			if((g_EndpointNum[alternate][i] & 0x80) == USB_EP_OUT){
				enable = USB_EP_INT_OUT_TRAN_ACK;
				disable = USB_EP_INT_OUT_TRAN_ERR | USB_EP_INT_IN_TRAN_ERR |
						  USB_EP_INT_OUT_TRAN_NAK | USB_EP_INT_IN_TRAN_NAK |
						  USB_EP_INT_IN_TRAN_ACK |
						  USB_EP_INT_OUT_SHORT_ACK;
			} else {
				enable = USB_EP_INT_IN_TRAN_ACK;
				disable = USB_EP_INT_OUT_TRAN_ERR | USB_EP_INT_IN_TRAN_ERR |
						  USB_EP_INT_OUT_TRAN_NAK | USB_EP_INT_IN_TRAN_NAK |
						  USB_EP_INT_OUT_TRAN_ACK |
						  USB_EP_INT_OUT_SHORT_ACK;
			}

			// Enable Interrupt
			UsbEndpointIntEnable(j, enable);
			// Disable Interrupt
			UsbEndpointIntDisable(j, disable);

			UsbEPrIntEnable(j);

			if(g_UsbState == USB_CONFIGURED_STATE){
				// Enable Endpoint
				UsbEnableEndpoint(j);

				EndpointCount++;
			}
		} else {
			// Disable Endpoint
			UsbDisableEndpoint(j);
		}
	}
#endif

	// Clear All FIFO
	UsbFIFOAllClear();

}
