/***************************************************************************
*
*	File name : usb_usb.c
*	Function  : USB driver (C17801)
*			Low level API.
*	Revision  :
*		August. 20, 2007	Yakushiji		First revision
*
****************************************************************************/

#include	"reg_801.h"
#include	"system.h"
#include	"usb_api.h"
#include	"usb_usb.h"
#include	"usb_request.h"
#include	"usb_control.h"
#include	"usb_bulk.h"

static unsigned short		DescriptorAddress;	// descriptor top address

// Prototype
unsigned char	UsbVBUSCheck(void);
unsigned char	UsbLineStateCheck(void);
int				UsbAttach(void);
void			UsbDetach(void);
void			UsbReset(void);
void			UsbSuspend(void);
void			UsbResume(void);
#ifdef	FOR_REMOTEWAKEUP
void			UsbRemoteWakeup(void);
#endif
int				UsbFIFORead(unsigned char, unsigned char *, unsigned short, unsigned short *);
#ifdef	FOR_BULK_TRANSFER
int				UsbFIFOWrite(unsigned char, unsigned char *, unsigned short, unsigned short *);
#endif
int				UsbFIFOReadRemain(unsigned char, unsigned short *);
int				UsbFIFOWriteRemain(unsigned char, unsigned short *);
int				UsbFIFOClear(unsigned char);
#ifndef	FOR_USB_MINIMUM_API
unsigned char	UsbGetFIFOReadJoin(void);
unsigned char	UsbGetFIFOWriteJoin(void);
#endif
#ifdef	FOR_TEST_MODE
int				UsbSetTestMode(unsigned char);
void			UsbStartTestMode(void);
#endif
void			UsbSetAutoAddress(unsigned char);
#ifndef	FOR_USB_MINIMUM_API
int				UsbSetOpMode(unsigned char);
#endif
void			UsbGetSetupPacket(T_USB_REQUEST_PACKET *);
#ifdef	FOR_BULK_TRANSFER
void			UsbEnableEndpoint(unsigned char);
void			UsbDisableEndpoint(unsigned char);
#endif
int				UsbSetEP0Control(unsigned char, unsigned char, unsigned char);
unsigned char	UsbGetEP0Control(unsigned char);
#ifdef	FOR_BULK_TRANSFER
int				UsbSetEndpointControl(unsigned char, unsigned char, unsigned char);
unsigned char	UsbGetEndpointControl(unsigned char);
#endif
int				UsbEndpointIntEnable(unsigned char, unsigned char);
int				UsbEndpointIntDisable(unsigned char, unsigned char);
#ifndef	FOR_USB_MINIMUM_API
unsigned char	UsbGetEndpointInt(unsigned char);
void			UsbClearEndpointInt(unsigned char, unsigned char);
#endif
void			UsbSetEndpointMaxSize(unsigned char, unsigned short);
#ifndef	FOR_USB_MINIMUM_API
unsigned short	UsbGetEndpointMaxSize(unsigned char);
#endif
#ifdef	FOR_BULK_TRANSFER
void			UsbSetEndpointNumber(unsigned char, unsigned char);
unsigned char	UsbGetEndpointNumber(unsigned char);
void			UsbSetEndpointDirection(unsigned char, unsigned char);
unsigned char	UsbGetEndpointDirection(unsigned char);
void			UsbSetEndpointMode(unsigned char, unsigned char);
unsigned char	UsbGetEndpointMode(unsigned char);
void			UsbSetEndpointToggleMode(unsigned char, unsigned char);
unsigned char	UsbGetEndpointToggleMode(unsigned char);
#endif
void			UsbSetFIFOArea(unsigned short *);
void			UsbGetFIFOArea(unsigned short *);
void			UsbWriteDescriptor(unsigned char *, unsigned short, unsigned short);
#ifndef	FOR_USB_MINIMUM_API
void			UsbReadDescriptor(unsigned char *, unsigned short, unsigned short);
#endif
void			UsbReplyDescriptor(unsigned short, unsigned short);
unsigned short	UsbRemainDescriptor(void);
unsigned char	UsbGetUSBState(void);

void			ActiveUSBIP(void);
#ifndef	FOR_SNOOZE_OFF
void			SnoozeUSBIP(void);
#endif
static void		InitializeModule(void);

unsigned char	g_UsbSnoozeFlag;

#ifdef	FOR_BULK_TRANSFER
unsigned char	g_EPChange[] = {0x00,0x01,0x02,0x04,0x08};
#endif


/*************************************************************
	UsbVBUSCheck()

	description:
		Get VBUS.

	argument	: None
	return		:
		USB_VBUS_ON	: VBUS is ON
		USB_VBUS_OFF: VBUS is OFF

**************************************************************/
unsigned char UsbVBUSCheck(void){

	int				i;
	volatile unsigned char	temp;

	// Read VBUS
	temp = (USB_USBStatus & USB_USBStatus_VBUS);

	// 5 count polling
	for(i = 0;i < 5; i++){

		if((USB_USBStatus & USB_USBStatus_VBUS) != temp){
			// Chattering prevention
			temp = (USB_USBStatus & USB_USBStatus_VBUS);

			// check continue
			i = 0;
		}
	}

	return temp;
}

/*************************************************************
	UsbLineStateCheck()

	description:
		Get Line state on USB cable.

	argument	: None
	return		:
		USB_LINE_SE1 : VBUS is ON
		USB_LINE_K   : VBUS is OFF
		USB_LINE_J   : VBUS is OFF
		USB_LINE_SE0 : VBUS is ON

**************************************************************/
unsigned char UsbLineStateCheck(void){

	int				i;
	volatile unsigned char	temp;

	// Read Line state
	temp = (USB_USBStatus & USB_USBStatus_LineState);

	// 5 count polling
	for(i = 0;i < 5; i++){

		if((USB_USBStatus & USB_USBStatus_LineState) != temp){
			// Chattering prevention
			temp = (USB_USBStatus & USB_USBStatus_LineState);

			// check continue
			i = 0;
		}
	}

	return temp;
}

/*************************************************************
	UsbAttach()

	description:
		USB attach process.

	argument	: None
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error complete

**************************************************************/
int UsbAttach(void){

	int		ret = ERROR;
	int		i;

	// VBUS check
	if(UsbVBUSCheck() == USB_VBUS_ON){
		/* VBUS ڑ */

#ifndef	FOR_SNOOZE_OFF
		// Active
		ActiveUSBIP();
#endif

		// if not Active then attach process
		if((USB_USBControl & USB_USBControl_ActiveUSB) != USB_USBControl_ActiveUSB){

			InitializeModule();

			g_ManagementFlag.DetachProg = 0;

			// Active USB & Disable BUS detect
			USB_USBControl |= (USB_USBControl_ActiveUSB | USB_USBControl_DisBusDetect);

			// Valid D+ pullup / Disable bitstuffing & NRZI Encoding
			USB_XcvrControl = (USB_XcvrControl & ~USB_XcvrControl_OpMode) |
							  (USB_XcvrControl_RpuEnb | USB_OPMODE_DISBITSTUFF);

			for(i = 0; i < 5; i++){
				// Chech Line State
				if(UsbLineStateCheck() == USB_LINE_J){
					// J State is success
					ret = SUCCESS;
					break;
				}

				// if VBUS changed then Detach
				if(g_ManagementFlag.VbusChange == 1){

					InterruptDisableUSB();
					g_ManagementFlag.VbusChange = 0;
					InterruptEnableUSB();

					break;
				}
			}

			// if Attach complete then
			if(ret == SUCCESS){
				// Interrupt enable for Reset & Suspend
				USB_SIEIntEnb |= (USB_SIEIntEnb_EnDetectReset | USB_SIEIntEnb_EnDetectSuspend);

#ifdef	FOR_NEGOTIATION
				// if negotiation is manual then valid BusDetect
				if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){
					// Valid Reset & Suspend
					USB_USBControl &= ~USB_USBControl_DisBusDetect;
				} else {
					// set auto negotiation mode
					USB_USBControl |= USB_USBControl_EnAutoNego;
				}
#else
				// set auto negotiation mode
				USB_USBControl |= USB_USBControl_EnAutoNego;
#endif

				// Clear flag
				g_ManagementFlag.Attach = 0;
			} else {
				// Detach
				UsbDetach();
				ret = ERROR;
			}
		} else {
			// Clear flag
			g_ManagementFlag.Attach = 0;
			ret = SUCCESS;
		}
	} else {
		// Detach
		UsbDetach();
		ret = ERROR;
	}

	return ret;
}


/*************************************************************
	UsbDetach()

	description:
		USB detach process.

	argument	: None
	return		: None

**************************************************************/
void UsbDetach(void){

	// if not Active then attach process
	if(g_ManagementFlag.DetachProg == 0){

#ifndef	FOR_SNOOZE_OFF
		// Active
		ActiveUSBIP();
#endif

		g_ManagementFlag.DetachProg = 1;

		// Invalid D+ pullup / Non drive
		USB_XcvrControl = (USB_XcvrControl & ~(USB_XcvrControl_OpMode | USB_XcvrControl_RpuEnb)) |
						   USB_OPMODE_NONDRIVE;

		// Inactive & Auto negotiation function off
		USB_USBControl &= ~(USB_USBControl_ActiveUSB | USB_USBControl_EnAutoNego);
	}

#ifndef	FOR_SNOOZE_OFF
	// Snooze
	SnoozeUSBIP();
#endif

	// Clear flag
	g_ManagementFlag.Detach = 0;
	g_ManagementFlag.Attach = 1;
}

/*************************************************************
	UsbReset()

	description:
		USB reset process.

	argument	: None
	return		: None

**************************************************************/
void UsbReset(void){

	InterruptDisableUSB();
	// Clear flag
	g_ManagementFlag.Reset = 0;
	InterruptEnableUSB();

	// Normal
	USB_XcvrControl = (USB_XcvrControl & ~USB_XcvrControl_OpMode) | USB_OPMODE_NORMAL;


#ifdef	FOR_NEGOTIATION
	// if negotiation is manual then invalid BusDetect
	if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){
		// Valid Reset & Suspend
		USB_USBControl |= USB_USBControl_DisBusDetect;
	}
#endif

	// Initialize Endpoint 0
	if(g_ManagementFlag.Init == 0){

		InitializeModule();
	}

#ifdef	FOR_NEGOTIATION
	// if negotiation is manual then start detect-J
	if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){
		USB_SIEIntStat  = USB_SIEIntStat_DetectJ;		// Clear detect-J
		USB_SIEIntEnb  |= USB_SIEIntEnb_EnDetectJ;		// Enable detect-J
		USB_USBControl |= USB_USBControl_StartDetectJ;	// Start detect-J
	}
#endif
}

/*************************************************************
	UsbSuspend()

	description:
		USB suspend process.

	argument	: None
	return		: None

**************************************************************/
void UsbSuspend(void){


	InterruptDisableUSB();
	// Clear flag
	g_ManagementFlag.Suspend = 0;
	InterruptEnableUSB();

#ifdef	FOR_NEGOTIATION
	// if negotiation is manual then suspend
	if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){
		USB_USBControl |= USB_USBControl_InSUSPEND;
	}
#endif

	// Clear Non-J interrupt
	USB_SIEIntStat = USB_SIEIntStat_NonJ;
	// Enable Non-J interrupt
	USB_SIEIntEnb |= USB_SIEIntEnb_EnNonJ;

#ifdef	FOR_NEGOTIATION
	// if negotiation is manual then invalid BusDetect
	if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){
		USB_USBControl |= (USB_USBControl & ~USB_USBControl_EnAutoNego) | USB_USBControl_DisBusDetect;
	}
#endif

#ifndef	FOR_SNOOZE_OFF
	// Call Snooze function
	SnoozeUSBIP();
#endif

	// Set resume flag
	g_ManagementFlag.Resume = 1;

}

/*************************************************************
	UsbResume()

	description:
		USB resume process.

	argument	: None
	return		: None

**************************************************************/
void UsbResume(void)
{
	unsigned char	lineState;

	// Change VBUS
	if(g_ManagementFlag.VbusChange == 1){
		InterruptDisableUSB();
		g_ManagementFlag.VbusChange = 0;
		InterruptEnableUSB();

		if(UsbVBUSCheck() == USB_VBUS_OFF){

			USB_USBControl &= ~USB_USBControl_InSUSPEND;

			// Disable Non-J interrupt
			USB_SIEIntEnb &= ~USB_SIEIntEnb_EnNonJ;

			UsbDetach();	// Detach

			// Clear flag
			g_ManagementFlag.Resume = 0;
		}
	} else {

#ifdef	FOR_REMOTEWAKEUP
		UsbRemoteWakeup();
#endif

		// Non-J interrupt factor
		if(g_ManagementFlag.NonJ == 1){

#ifndef	FOR_SNOOZE_OFF
			// Call Active function
			ActiveUSBIP();
#endif

			lineState = UsbLineStateCheck();

			InterruptDisableUSB();
			g_ManagementFlag.NonJ = 0;
			InterruptEnableUSB();

			if(lineState != USB_LINE_J && lineState != USB_LINE_SE1){

				USB_USBControl &= ~USB_USBControl_InSUSPEND;

#ifdef	FOR_NEGOTIATION
				// if negotiation is manual then invalid BusDetect
				if(g_Config.negotiationMode == USB_NEGOTIATION_MANUAL){

					if( lineState == USB_LINE_SE0 ){
						UsbReset();		// USB Reset
					}

					if(lineState == USB_LINE_K){
						// USB Resume
						USB_SIEIntStat  = USB_SIEIntStat_DetectJ;		// Clear detect-J
						USB_SIEIntEnb  |= USB_SIEIntEnb_EnDetectJ;		// Enable detect-J
						USB_USBControl |= USB_USBControl_StartDetectJ;	// Start detect-J

					}
				}
#endif

				// Clear flag
				g_ManagementFlag.Resume = 0;
			} else {

#ifndef	FOR_SNOOZE_OFF
				// Call Active function
				SnoozeUSBIP();
#endif

				// Enable Non-J interrupt
				USB_SIEIntEnb |= USB_SIEIntEnb_EnNonJ;
			}
		}
	}
}

#ifdef	FOR_REMOTEWAKEUP
/*************************************************************
	UsbRemoteWakeup()

	description:
		USB remote wakeup process.

	argument	: None
	return		: None

**************************************************************/
void UsbRemoteWakeup(void){

	char		remoteWakeup;

	remoteWakeup = 0;

	// check valid remote wakeup enable
	if(g_ManagementFlag.EnbRemoteWakeup){
		// check RemoteWakeup
		if(g_Config.remoteFunc() == SUCCESS){
			remoteWakeup = 1;

			Timer0Start();
			while(Timer0Check(TMOUT_5) != TIMEOUT){
				if(g_ManagementFlag.VbusChange || g_ManagementFlag.NonJ)){
					remoteWakeup = 0;
					break;
				}
			}
			Timer0Stop();
		}
	}

	// valid remoteWakeup
	if(remoteWakeup){
		remoteWakeup = 0;

		// Disable Non-J interrupt
		UsbSIEIntDisable(USB_SIE_INT_NONJ);

		// 1msec check
		Timer0Start();

		USB_USBControl |= USB_USBControl_SendWakeup;

		while(Timer0Check(TMOUT_1) != TIMEOUT);
		Timer0Stop();	/* ^C}[~ */

		USB_USBControl &= ~USB_USBControl_SendWakeup;

		InterruptDisableUSB();

		// Set quasi Non-J
		g_ManagementFlag.NonJ = 1;

		InterruptEnableUSB();
	}
}
#endif	// FOR_REMOTEWAKEUP

/*************************************************************
	UsbFIFORead()

	description:
		Read FIFO.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		buffer	: Buffer pointer
		requestRength : Request length
		actualLength  : Read actual length
	return		:
		SUCCESS	: Nomal complete
		ERROR	: Agument error

**************************************************************/
int	UsbFIFORead(unsigned char number,unsigned char *buffer,
				unsigned short requestLength, unsigned short *actualLength){

	int				ret = ERROR;
	unsigned short	fifoRemain;
	int				i;

#ifdef	FOR_BULK_TRANSFER
	// Set Join Endpoint
	USB_CPUJoinRd = g_EPChange[number];

	// Initialize actual length
	*actualLength = 0;

	// Set value and Endpoint number check
	if(USB_CPUJoinRd == g_EPChange[number]){
		// check buffer pointer & length
		if(buffer != NULL && requestLength != 0){

			// Get FIFO Remain
			((unsigned char *)&fifoRemain)[1] = USB_EPnRdRemainH;
			((unsigned char *)&fifoRemain)[0] = USB_EPnRdRemainL;

			// Valid FIFO data
			if(fifoRemain != 0){
				// Set FIFO read access
				USB_EnEPnFIFOAccess |= USB_EnEPnFIFOAccess_EnEPnFIFORd;

				// Read FIFO Data
				for(i = 0; i < requestLength && fifoRemain != 0; i++, fifoRemain--){
					*buffer++ = USB_EPnFIFOforCPU;
				}

				// Clear FIFO read access
				USB_EnEPnFIFOAccess &= ~USB_EnEPnFIFOAccess_EnEPnFIFORd;

				// Set Actual data length
				*actualLength = i;
			}

			ret = SUCCESS;
		}
	}
#else
	if(number == USB_ENDPOINT_0){
		// Set Join Endpoint
		USB_CPUJoinRd = USB_ENDPOINT_0;

		// Initialize actual length
		*actualLength = 0;

		// Set value and Endpoint number check
		if(USB_CPUJoinRd == USB_ENDPOINT_0){
			// check buffer pointer & length
			if(buffer != NULL && requestLength != 0){

				// Get FIFO Remain
				((unsigned char *)&fifoRemain)[1] = USB_EPnRdRemainH;
				((unsigned char *)&fifoRemain)[0] = USB_EPnRdRemainL;

				// Valid FIFO data
				if(fifoRemain != 0){
					// Set FIFO read access
					USB_EnEPnFIFOAccess |= USB_EnEPnFIFOAccess_EnEPnFIFORd;

					// Read FIFO Data
					for(i = 0; i < requestLength && fifoRemain != 0; i++, fifoRemain--){
						*buffer++ = USB_EPnFIFOforCPU;
					}

					// Clear FIFO read access
					USB_EnEPnFIFOAccess &= ~USB_EnEPnFIFOAccess_EnEPnFIFORd;

					// Set Actual data length
					*actualLength = i;
				}

				ret = SUCCESS;
			}
		}
	}
#endif

	return ret;
}

/*************************************************************
	UsbFIFOWrite()

	description:
		Write FIFO.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		buffer	: Buffer pointer
		requestRength : Request length
		actualLength  : Write actual length
	return		:
		SUCCESS	: Nomal complete
		ERROR	: Agument error

**************************************************************/
int	UsbFIFOWrite(unsigned char number,unsigned char *buffer,
				 unsigned short requestLength, unsigned short *actualLength){

	int				ret = ERROR;
	unsigned short	fifoRemain;
	int				i;

#ifdef	FOR_BULK_TRANSFER
	// Set Join Endpoint
	USB_CPUJoinWr = g_EPChange[number];

	// Initialize actual length
	*actualLength = 0;

	// Set value and Endpoint number check
	if(USB_CPUJoinWr == g_EPChange[number]){
		// check buffer pointer & length
		if(buffer != NULL && requestLength != 0){

			// Get FIFO Remain
			((unsigned char *)&fifoRemain)[1] = USB_EPnWrRemainH;
			((unsigned char *)&fifoRemain)[0] = USB_EPnWrRemainL;

			// Valid FIFO data
			if(fifoRemain != 0){
				// Set FIFO write access
				USB_EnEPnFIFOAccess |= USB_EnEPnFIFOAccess_EnEPnFIFOWr;

				// Read FIFO Data
				for(i = 0; i < requestLength && fifoRemain != 0; i++, fifoRemain--){
					USB_EPnFIFOforCPU = *buffer++;
				}

				// Clear FIFO write access
				USB_EnEPnFIFOAccess &= ~USB_EnEPnFIFOAccess_EnEPnFIFOWr;

				// Set Actual data length
				*actualLength = i;
			}

			ret = SUCCESS;
		}
	}
#else
	if(number == USB_ENDPOINT_0){
		// Set Join Endpoint
		USB_CPUJoinWr = USB_ENDPOINT_0;

		// Initialize actual length
		*actualLength = 0;

		// Set value and Endpoint number check
		if(USB_CPUJoinWr == USB_ENDPOINT_0){
			// check buffer pointer & length
			if(buffer != NULL && requestLength != 0){

				// Get FIFO Remain
				((unsigned char *)&fifoRemain)[1] = USB_EPnWrRemainH;
				((unsigned char *)&fifoRemain)[0] = USB_EPnWrRemainL;

				// Valid FIFO data
				if(fifoRemain != 0){
					// Set FIFO write access
					USB_EnEPnFIFOAccess |= USB_EnEPnFIFOAccess_EnEPnFIFOWr;

					// Read FIFO Data
					for(i = 0; i < requestLength && fifoRemain != 0; i++, fifoRemain--){
						USB_EPnFIFOforCPU = *buffer++;
					}

					// Clear FIFO write access
					USB_EnEPnFIFOAccess &= ~USB_EnEPnFIFOAccess_EnEPnFIFOWr;

					// Set Actual data length
					*actualLength = i;
				}

				ret = SUCCESS;
			}
		}
	}
#endif

	return ret;
}

/*************************************************************
	UsbFIFOReadRemain()

	description:
		Get Read FIFO remain.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		remain	: Read FIFO remain
	return		:
		SUCCESS	: Nomal complete
		ERROR	: Agument error

**************************************************************/
int	UsbFIFOReadRemain(unsigned char number, unsigned short *remain){

	int				ret = ERROR;

#ifdef	FOR_BULK_TRANSFER
	// Set Join Endpoint
	USB_CPUJoinRd = g_EPChange[number];

	// Initialize remain
	*remain = 0;

	// Set value and Endpoint number check
	if(USB_CPUJoinRd == g_EPChange[number]){
		// Get FIFO Remain
		((unsigned char *)remain)[1] = USB_EPnRdRemainH;
		((unsigned char *)remain)[0] = USB_EPnRdRemainL;

		ret = SUCCESS;
	}
#else
	if(number == USB_ENDPOINT_0){
		// Set Join Endpoint
		USB_CPUJoinRd = USB_ENDPOINT_0;

		// Initialize remain
		*remain = 0;

		// Set value and Endpoint number check
		if(USB_CPUJoinRd == USB_ENDPOINT_0){
			// Get FIFO Remain
			((unsigned char *)remain)[1] = USB_EPnRdRemainH;
			((unsigned char *)remain)[0] = USB_EPnRdRemainL;

			ret = SUCCESS;
		}
	}
#endif
	return ret;
}

#ifdef	FOR_BULK_TRANSFER
/*************************************************************
	UsbFIFOWriteRemain()

	description:
		Get write FIFO remain.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		remain	: write FIFO remain
	return		:
		SUCCESS	: Nomal complete
		ERROR	: Agument error

**************************************************************/
int	UsbFIFOWriteRemain(unsigned char number, unsigned short *remain){

	int				ret = ERROR;

	// Set Join Endpoint
	USB_CPUJoinWr = g_EPChange[number];

	// Initialize remain
	*remain = 0;

	// Set value and Endpoint number check
	if(USB_CPUJoinWr == g_EPChange[number]){
		// Get FIFO Remain
		((unsigned char *)remain)[1] = USB_EPnWrRemainH;
		((unsigned char *)remain)[0] = USB_EPnWrRemainL;

		ret = SUCCESS;
	}

	return ret;
}
#endif

/*************************************************************
	UsbFIFOClear()

	description:
		FIFO Clear.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		SUCCESS	: Nomal complete
		ERROR	: Agument error

**************************************************************/
int	UsbFIFOClear(unsigned char number){

	int				ret = ERROR;

#ifdef	FOR_BULK_TRANSFER
	// Set Join Endpoint
	USB_CPUJoinWr = g_EPChange[number];

	// Set value and Endpoint number check
	if(USB_CPUJoinWr == g_EPChange[number]){

		if(number == USB_ENDPOINT_0){
			USB_EPnControl |= USB_EPnControl_EP0FIFOClr;
		} else {
			USB_EPrFIFOClr = g_EPChange[number];
		}
		ret = SUCCESS;
	}
#else
	if(number == USB_ENDPOINT_0){
		// Set Join Endpoint
		USB_CPUJoinWr = USB_ENDPOINT_0;

		// Set value and Endpoint number check
		if(USB_CPUJoinWr == USB_ENDPOINT_0){

			USB_EPnControl |= USB_EPnControl_EP0FIFOClr;

			ret = SUCCESS;
		}
	}
#endif

	return ret;
}

#ifndef	FOR_USB_MINIMUM_API
/*************************************************************
	UsbGetFIFOReadJoin()

	description:
		Get join to read FIFO.

	argument	: None
	return		:
				USB_ENDPOINT_0
				USB_ENDPOINT_A
				USB_ENDPOINT_B
				USB_ENDPOINT_C
				USB_ENDPOINT_D

**************************************************************/
unsigned char	UsbGetFIFOReadJoin(void){

	int				i;
	unsigned char	join;

	join = USB_CPUJoinRd;

	for(i = 0; i < 4; i++){
		if(join == g_EPChange[i]){
			break;
		}
	}

	return i;
}

/*************************************************************
	UsbGetFIFOWriteJoin()

	description:
		Get join to write FIFO.

	argument	: None
	return		:
				USB_ENDPOINT_0
				USB_ENDPOINT_A
				USB_ENDPOINT_B
				USB_ENDPOINT_C
				USB_ENDPOINT_D

**************************************************************/
unsigned char	UsbGetFIFOWriteJoin(void){

	int				i;
	unsigned char	join;

	join = USB_CPUJoinWr;

	for(i = 0; i < 4; i++){
		if(join == g_EPChange[i]){
			break;
		}
	}

	return i;
}
#endif

#ifdef	FOR_TEST_MODE
/*************************************************************
	UsbSetTestMode()

	description:
		Set Test mode.

	argument	:
		testMode: Test mode
					USB_TESTMODE_J
					USB_TESTMODE_K
					USB_TESTMODE_SE0_NAK
					USB_TESTMODE_PACKET
	return		:
		SUCCESS	: Nomal complete
		ERROR	: Agument error

**************************************************************/
int	UsbSetTestMode(unsigned char testMode){

	int				ret = ERROR;
	int				i;
	int				count;

	// check test mode
	for(i = 0, count = 0; i < 4; i++){
		if((testMode & (1 << i)) != 0){
			count++;
		}
	}

	// select test only one
	if(count == 1 && (testMode & 0xF0) == 0){

		// set test mode
		USB_USBTest = testMode;

		ret = SUCCESS;
	}

	return ret;
}

/*************************************************************
	UsbStartTestMode()

	description:
		Set Test mode.

	argument	: None
	return		: None

**************************************************************/
void UsbStartTestMode(void){

	// Disable interrupt
	USB_MainIntEnb	= 0;
	USB_SIEIntEnb	= 0;
	USB_EPrIntEnb	= 0;
	USB_FIFOIntEnb	= 0;

	// if Test J or Test K then set operation mode
	if((USB_USBTest & USB_USBTest_TestJ) || (USB_USBTest  & USB_USBTest_TestK)){

		// Disable Bitstuffing and NRZI encoding
		USB_XcvrControl = (USB_XcvrControl & ~USB_XcvrControl_OpMode) | USB_OPMODE_DISBITSTUFF;
	}

	// clear auto negotiation mode
	USB_USBControl &= ~USB_USBControl_EnAutoNego;
	// Disable Reset & Suspend
	USB_USBControl |= USB_USBControl_DisBusDetect;

	// Start Test mode
	USB_USBTest |= USB_USBTest_EnUSBTest;

	// Wait VBUS off
	while(1){
		// break VBUS off
		if(UsbVBUSCheck() == USB_VBUS_OFF){
			break;
		}
	}

	// Stop Test mode
	USB_USBTest &= ~USB_USBTest_EnUSBTest;

	// Disable D+ pullup &  Non Drive
	USB_XcvrControl = (USB_XcvrControl & ~(USB_XcvrControl_OpMode | USB_XcvrControl_RpuEnb)) |
					  USB_OPMODE_NONDRIVE;

	// Inactive
	USB_USBControl &= ~USB_USBControl_ActiveUSB;

	// Wait chip reset
	while(1){
		asm("halt");
	}
}
#endif

/*************************************************************
	UsbSetAutoAddress()

	description:
		Valid or Invalid Auto Address mode.

	argument	:
		mode	: set mode
					USB_AUTOADDRESS_ON
					USB_AUTOADDRESS_OFF
	return		: None

**************************************************************/
void UsbSetAutoAddress(unsigned char mode){

	if(mode == USB_AUTOADDRESS_ON){
		USB_USBAddress |= mode;
	} else {
		USB_USBAddress &= ~USB_AUTOADDRESS_ON;
	}
}

#ifndef	FOR_USB_MINIMUM_API
/*************************************************************
	UsbSetOpMode()

	description:
		Set Transceiver Macro operation mode.

	argument	:
		opMode	: oparation mode
					USB_OPMODE_NORMAL
					USB_OPMODE_NONDRIVE
					USB_OPMODE_DISBITSTUFF
	return		:
		SUCCESS	: Nomal complete
		ERROR	: Agument error

**************************************************************/
int UsbSetOpMode(unsigned char opMode){

	int		ret = ERROR;

	if(opMode == USB_OPMODE_NORMAL ||
	   opMode == USB_OPMODE_NONDRIVE ||
	   opMode == USB_OPMODE_DISBITSTUFF){
		USB_XcvrControl = (USB_XcvrControl & ~USB_XcvrControl_OpMode) | opMode;

		ret = SUCCESS;
	}

	return ret;
}
#endif

/*************************************************************
	UsbGetSetupPacket()

	description:
		Get setup packet.

	argument	:
		packet	: Request packet pointer
	return		: None

**************************************************************/
void UsbGetSetupPacket(T_USB_REQUEST_PACKET *packet){

	// Read Request type
	packet->requestType = USB_EP0Setup0;

	// Read Request number
	packet->request = USB_EP0Setup1;

	// Read Value
	((unsigned char *)&packet->value)[0] = USB_EP0Setup2;
	((unsigned char *)&packet->value)[1] = USB_EP0Setup3;

	// Read Index
	((unsigned char *)&packet->index)[0] = USB_EP0Setup4;
	((unsigned char *)&packet->index)[1] = USB_EP0Setup5;

	// Read Length
	((unsigned char *)&packet->length)[0] = USB_EP0Setup6;
	((unsigned char *)&packet->length)[1] = USB_EP0Setup7;
}

#ifdef	FOR_BULK_TRANSFER
/*************************************************************
	UsbEnableEndpoint()

	description:
		Enable endpoint

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		: None

**************************************************************/
void UsbEnableEndpoint(unsigned char number){

	switch(number){
	case USB_ENDPOINT_A:
		USB_EPaConfig0 |= USB_EPaConfig0_EnEndPoint;
		break;
	case USB_ENDPOINT_B:
		USB_EPbConfig0 |= USB_EPbConfig0_EnEndPoint;
		break;
	case USB_ENDPOINT_C:
		USB_EPcConfig0 |= USB_EPcConfig0_EnEndPoint;
		break;
	case USB_ENDPOINT_D:
		USB_EPdConfig0 |= USB_EPdConfig0_EnEndPoint;
		break;
	}
}

/*************************************************************
	UsbDisableEndpoint()

	description:
		Disable endpoint

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		: None

**************************************************************/
void UsbDisableEndpoint(unsigned char number){

	switch(number){
	case USB_ENDPOINT_A:
		USB_EPaConfig0 &= ~USB_EPaConfig0_EnEndPoint;
		break;
	case USB_ENDPOINT_B:
		USB_EPbConfig0 &= ~USB_EPbConfig0_EnEndPoint;
		break;
	case USB_ENDPOINT_C:
		USB_EPcConfig0 &= ~USB_EPcConfig0_EnEndPoint;
		break;
	case USB_ENDPOINT_D:
		USB_EPdConfig0 &= ~USB_EPdConfig0_EnEndPoint;
		break;
	}
}
#endif

/*************************************************************
	UsbSetEP0Control()

	description:
		Set endpoint control.

	argument	:
		direction: EP0 direction
					USB_EP0_IN
					USB_EP0_OUT
		set		: Set bit
					USB_EP_CTL_STALL
					USB_EP_CTL_NAK
					USB_EP_CTL_TOGGLE_CLR
					USB_EP_CTL_TOGGLE_SET
					USB_EP_CTL_DISAFNAK
					USB_EP_CTL_ENSHORTPKT
					USB_EP_CTL_AUTOFORCENAK
		clear	: Clear bit
					USB_EP_CTL_STALL
					USB_EP_CTL_NAK
					USB_EP_CTL_DISAFNAK
					USB_EP_CTL_ENSHORTPKT
					USB_EP_CTL_AUTOFORCENAK
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error complete

**************************************************************/
int UsbSetEP0Control(unsigned char direction, unsigned char set, unsigned char clear){

	int		ret = ERROR;

	// The same Bit isn't set.
	if((set & clear) == 0 && (clear & (USB_EP_CTL_TOGGLE_CLR | USB_EP_CTL_TOGGLE_SET)) == 0){

		if(direction == USB_EP0_IN){
			USB_EP0ControlIN  = ((USB_EP0ControlIN & ~clear) | set);
		} else {
			USB_EP0ControlOUT = ((USB_EP0ControlOUT & ~clear) | set);
		}
		ret = SUCCESS;
	}

	return ret;
}

/*************************************************************
	UsbGetEP0Control()

	description:
		Get Endpoint control value.

	argument	:
		direction: Control direction
					USB_EP0_IN
					USB_EP0_OUT
	return		:
		USB_EP_CTL_STALL
		USB_EP_CTL_NAK
		USB_EP_CTL_TOGGLE_0
		USB_EP_CTL_TOGGLE_1
		USB_EP_CTL_ENSHORTPKT
		USB_EP_CTL_AUTOFORCENAK

**************************************************************/
unsigned char UsbGetEP0Control(unsigned char direction){

	unsigned char	state;

	if(direction == USB_EP0_IN){
		state = USB_EP0ControlIN;
	} else {
		state = USB_EP0ControlOUT;
	}

	return state;
}

#ifdef	FOR_BULK_TRANSFER
/*************************************************************
	UsbSetEndpointControl()

	description:
		Set endpoint control.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		set		: Set bit
					USB_EP_CTL_STALL
					USB_EP_CTL_NAK
					USB_EP_CTL_TOGGLE_CLR
					USB_EP_CTL_TOGGLE_SET
					USB_EP_CTL_DISAFNAK
					USB_EP_CTL_ENSHORTPKT
					USB_EP_CTL_AUTOFORCENAK
		clear	: Clear bit
					USB_EP_CTL_STALL
					USB_EP_CTL_NAK
					USB_EP_CTL_DISAFNAK
					USB_EP_CTL_ENSHORTPKT
					USB_EP_CTL_AUTOFORCENAK
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error complete

**************************************************************/
int UsbSetEndpointControl(unsigned char number, unsigned char set, unsigned char clear){

	int		ret = ERROR;

	// The same Bit isn't set.
	if((set & clear) == 0 &&
	   (clear & (USB_EP_CTL_TOGGLE_CLR | USB_EP_CTL_TOGGLE_SET)) == 0){

		switch(number){
		case USB_ENDPOINT_A:
			USB_EPaControl = ((USB_EPaControl & ~clear) | set);
			ret = SUCCESS;
			break;
		case USB_ENDPOINT_B:
			USB_EPbControl = ((USB_EPbControl & ~clear) | set);
			ret = SUCCESS;
			break;
		case USB_ENDPOINT_C:
			USB_EPcControl = ((USB_EPcControl & ~clear) | set);
			ret = SUCCESS;
			break;
		case USB_ENDPOINT_D:
			USB_EPdControl = ((USB_EPdControl & ~clear) | set);
			ret = SUCCESS;
			break;
		}
	}

	return ret;
}

/*************************************************************
	UsbGetEndpointControl()

	description:
		Get Endpoint control value.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		USB_EP_CTL_STALL
		USB_EP_CTL_NAK
		USB_EP_CTL_TOGGLE_0
		USB_EP_CTL_TOGGLE_1
		USB_EP_CTL_DISAFNAK
		USB_EP_CTL_ENSHORTPKT
		USB_EP_CTL_AUTOFORCENAK

**************************************************************/
unsigned char UsbGetEndpointControl(unsigned char number){

	unsigned char	state;

	switch(number){
	case USB_ENDPOINT_A:
		state = USB_EPaControl;
		break;
	case USB_ENDPOINT_B:
		state = USB_EPbControl;
		break;
	case USB_ENDPOINT_C:
		state = USB_EPcControl;
		break;
	case USB_ENDPOINT_D:
		state = USB_EPdControl;
		break;
	}

	return state;
}
#endif

#ifdef	FOR_BULK_TRANSFER
/*************************************************************
	UsbEndpointIntEnable()

	description:
		Enable endpoint Interrupt.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		event	: Enable event
					USB_EP_INT_OUT_TRAN_ERR
					USB_EP_INT_IN_TRAN_ERR
					USB_EP_INT_OUT_TRAN_NAK
					USB_EP_INT_IN_TRAN_NAK
					USB_EP_INT_OUT_TRAN_ACK
					USB_EP_INT_IN_TRAN_ACK
					USB_EP_INT_OUT_SHORT_ACK
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error complete

**************************************************************/
int UsbEndpointIntEnable(unsigned char number, unsigned char event){

	int		ret = ERROR;

	// Valid event value
	if((event & 0x7F)){
		switch(number){
		case USB_ENDPOINT_0:
			if((event & USB_EP_INT_OUT_SHORT_ACK) == 0){
				USB_EP0IntEnb |= event;

				ret = SUCCESS;
			}
			break;
		case USB_ENDPOINT_A:
			USB_EPaIntEnb |= event;
			ret = SUCCESS;
			break;
		case USB_ENDPOINT_B:
			USB_EPbIntEnb |= event;
			ret = SUCCESS;
			break;
		case USB_ENDPOINT_C:
			USB_EPcIntEnb |= event;
			ret = SUCCESS;
			break;
		case USB_ENDPOINT_D:
			USB_EPdIntEnb |= event;
			ret = SUCCESS;
			break;
		}
	}

	return ret;
}
#endif

/*************************************************************
	UsbEndpointIntDisable()

	description:
		Disable endpoint interrupt.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		event	: Enable event
					USB_EP_INT_OUT_TRAN_ERR
					USB_EP_INT_IN_TRAN_ERR
					USB_EP_INT_OUT_TRAN_NAK
					USB_EP_INT_IN_TRAN_NAK
					USB_EP_INT_OUT_TRAN_ACK
					USB_EP_INT_IN_TRAN_ACK
					USB_EP_INT_OUT_SHORT_ACK
	return		:
		SUCCESS	: Normal complete
		ERROR	: Error complete

**************************************************************/
int UsbEndpointIntDisable(unsigned char number, unsigned char event){

	int		ret = ERROR;

	// Valid event value
	if((event & 0x7F)){
		switch(number){
		case USB_ENDPOINT_0:
			if((event & USB_EP_INT_OUT_SHORT_ACK) == 0){
				USB_EP0IntEnb &= ~event;

				ret = SUCCESS;
			}
			break;
#ifdef	FOR_BULK_TRANSFER
		case USB_ENDPOINT_A:
			USB_EPaIntEnb &= ~event;
			ret = SUCCESS;
			break;
		case USB_ENDPOINT_B:
			USB_EPbIntEnb &= ~event;
			ret = SUCCESS;
			break;
		case USB_ENDPOINT_C:
			USB_EPcIntEnb &= ~event;
			ret = SUCCESS;
			break;
		case USB_ENDPOINT_D:
			USB_EPdIntEnb &= ~event;
			ret = SUCCESS;
			break;
#endif
		}
	}

	return ret;
}

#ifndef	FOR_USB_MINIMUM_API
/*************************************************************
	UsbGetEndpointInt()

	description:
		Get endpoint Interrupt factor.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		event	: Enable event
	return		:
		USB_EP_INT_OUT_TRAN_ERR
		USB_EP_INT_IN_TRAN_ERR
		USB_EP_INT_OUT_TRAN_NAK
		USB_EP_INT_IN_TRAN_NAK
		USB_EP_INT_OUT_TRAN_ACK
		USB_EP_INT_IN_TRAN_ACK
		USB_EP_INT_OUT_SHORT_ACK

**************************************************************/
unsigned char UsbGetEndpointInt(unsigned char number){

	unsigned char		intState;

	switch(number){
	case USB_ENDPOINT_0:
		intState = USB_EP0IntStat;
		break;
#ifdef	FOR_BULK_TRANSFER
	case USB_ENDPOINT_A:
		intState = USB_EPaIntStat;
		break;
	case USB_ENDPOINT_B:
		intState = USB_EPbIntStat;
		break;
	case USB_ENDPOINT_C:
		intState = USB_EPcIntStat;
		break;
	case USB_ENDPOINT_D:
		intState = USB_EPdIntStat;
		break;
#endif
	}

	return intState;
}

/*************************************************************
	UsbClearEndpointInt()

	description:
		Clear endpoint Interrupt factor.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		event	: Clear event
					USB_EP_INT_OUT_TRAN_ERR
					USB_EP_INT_IN_TRAN_ERR
					USB_EP_INT_OUT_TRAN_NAK
					USB_EP_INT_IN_TRAN_NAK
					USB_EP_INT_OUT_TRAN_ACK
					USB_EP_INT_IN_TRAN_ACK
					USB_EP_INT_OUT_SHORT_ACK
	return		: None

**************************************************************/
void UsbClearEndpointInt(unsigned char number, unsigned char event){

	switch(number){
	case USB_ENDPOINT_0:
		USB_EP0IntStat = event;
		break;
#ifdef	FOR_BULK_TRANSFER
	case USB_ENDPOINT_A:
		USB_EPaIntStat = event;
		break;
	case USB_ENDPOINT_B:
		USB_EPbIntStat = event;
		break;
	case USB_ENDPOINT_C:
		USB_EPcIntStat = event;
		break;
	case USB_ENDPOINT_D:
		USB_EPdIntStat = event;
		break;
#endif
	}
}
#endif

/*************************************************************
	UsbSetEndpointMaxSize()

	description:
		Set endpoint Max packet size.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		size	: Max packet size
	return		: None

**************************************************************/
void UsbSetEndpointMaxSize(unsigned char number, unsigned short size){

	switch(number){
	case USB_ENDPOINT_0:
		USB_EP0MaxSize = (unsigned char)(size & USB_EP0MaxSize_EP0MaxSize);
		break;
#ifdef	FOR_BULK_TRANSFER
	case USB_ENDPOINT_A:
		USB_EPaMaxSizeH = (((unsigned char *)&size)[1] & USB_EPaMaxSizeH_EPaMaxSize);
		USB_EPaMaxSizeL = (((unsigned char *)&size)[0] & USB_EPaMaxSizeL_EPaMaxSize);
		break;
	case USB_ENDPOINT_B:
		USB_EPbMaxSizeH = (((unsigned char *)&size)[1] & USB_EPbMaxSizeH_EPbMaxSize);
		USB_EPbMaxSizeL = (((unsigned char *)&size)[0] & USB_EPbMaxSizeL_EPbMaxSize);
		break;
	case USB_ENDPOINT_C:
		USB_EPcMaxSizeH = (((unsigned char *)&size)[1] & USB_EPcMaxSizeH_EPcMaxSize);
		USB_EPcMaxSizeL = (((unsigned char *)&size)[0] & USB_EPcMaxSizeL_EPcMaxSize);
		break;
	case USB_ENDPOINT_D:
		USB_EPdMaxSizeH = (((unsigned char *)&size)[1] & USB_EPdMaxSizeH_EPdMaxSize);
		USB_EPdMaxSizeL = (((unsigned char *)&size)[0] & USB_EPdMaxSizeL_EPdMaxSize);
		break;
#endif
	}
}

#ifndef	FOR_USB_MINIMUM_API
/*************************************************************
	UsbGetEndpointMaxSize()

	description:
		Get endpoint Max packet size.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_0
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		0xFFFF	: Error
		etc		: Read value

**************************************************************/
unsigned short UsbGetEndpointMaxSize(unsigned char number){

	unsigned short	size;

	size = 0xFFFF;

	switch(number){
	case USB_ENDPOINT_0:
		size = (USB_EP0MaxSize & USB_EP0MaxSize_EP0MaxSize);
		break;
#ifdef	FOR_BULK_TRANSFER
	case USB_ENDPOINT_A:
		((unsigned char *)&size)[1] = (USB_EPaMaxSizeH & USB_EPaMaxSizeH_EPaMaxSize);
		((unsigned char *)&size)[0] = (USB_EPaMaxSizeL & USB_EPaMaxSizeL_EPaMaxSize);
		break;
	case USB_ENDPOINT_B:
		((unsigned char *)&size)[1] = (USB_EPbMaxSizeH & USB_EPbMaxSizeH_EPbMaxSize);
		((unsigned char *)&size)[0] = (USB_EPbMaxSizeL & USB_EPbMaxSizeL_EPbMaxSize);
		break;
	case USB_ENDPOINT_C:
		((unsigned char *)&size)[1] = (USB_EPcMaxSizeH & USB_EPcMaxSizeH_EPcMaxSize);
		((unsigned char *)&size)[0] = (USB_EPcMaxSizeL & USB_EPcMaxSizeL_EPcMaxSize);
		break;
	case USB_ENDPOINT_D:
		((unsigned char *)&size)[1] = (USB_EPdMaxSizeH & USB_EPdMaxSizeH_EPdMaxSize);
		((unsigned char *)&size)[0] = (USB_EPdMaxSizeL & USB_EPdMaxSizeL_EPdMaxSize);
		break;
#endif
	}

	return size;
}
#endif

#ifdef	FOR_BULK_TRANSFER
/*************************************************************
	UsbSetEndpointNumber()

	description:
		Set endpoint number.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		setNumber : Endpoint set number
	return		: None

**************************************************************/
void UsbSetEndpointNumber(unsigned char number, unsigned char setNumber){

	switch(number){
	case USB_ENDPOINT_A:
		USB_EPaConfig0 = (USB_EPaConfig0 & ~USB_EPaConfig0_EndPointNumber) |
							(setNumber & USB_EPaConfig0_EndPointNumber);
		break;
	case USB_ENDPOINT_B:
		USB_EPbConfig0 = (USB_EPbConfig0 & ~USB_EPbConfig0_EndPointNumber) |
							(setNumber & USB_EPbConfig0_EndPointNumber);
		break;
	case USB_ENDPOINT_C:
		USB_EPcConfig0 = (USB_EPcConfig0 & ~USB_EPcConfig0_EndPointNumber) |
							(setNumber & USB_EPcConfig0_EndPointNumber);
		break;
	case USB_ENDPOINT_D:
		USB_EPdConfig0 = (USB_EPdConfig0 & ~USB_EPdConfig0_EndPointNumber) |
							(setNumber & USB_EPdConfig0_EndPointNumber);
		break;
	}
}

/*************************************************************
	UsbGetEndpointNumber()

	description:
		Get endpoint number.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		0xFF	: Error
		etc		: Endpoint set number

**************************************************************/
unsigned char UsbGetEndpointNumber(unsigned char number){

	unsigned char	setNumber;

	setNumber = 0xFF;

	switch(number){
	case USB_ENDPOINT_A:
		setNumber = (USB_EPaConfig0 & USB_EPaConfig0_EndPointNumber);
		break;
	case USB_ENDPOINT_B:
		setNumber = (USB_EPbConfig0 & USB_EPbConfig0_EndPointNumber);
		break;
	case USB_ENDPOINT_C:
		setNumber = (USB_EPcConfig0 & USB_EPcConfig0_EndPointNumber);
		break;
	case USB_ENDPOINT_D:
		setNumber = (USB_EPdConfig0 & USB_EPdConfig0_EndPointNumber);
		break;
	}

	return setNumber;
}

/*************************************************************
	UsbSetEndpointDirection()

	description:
		Set endpoint direction.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		direction : Endpoint direction
					USB_EP_IN
					USB_EP_OUT
	return		: None

**************************************************************/
void UsbSetEndpointDirection(unsigned char number, unsigned char direction){

	switch(number){
	case USB_ENDPOINT_A:
		USB_EPaConfig0 = (USB_EPaConfig0 & ~USB_EPaConfig0_INxOUT) |
							(direction & USB_EPaConfig0_INxOUT);
		break;
	case USB_ENDPOINT_B:
		USB_EPbConfig0 = (USB_EPbConfig0 & ~USB_EPbConfig0_INxOUT) |
							(direction & USB_EPbConfig0_INxOUT);
		break;
	case USB_ENDPOINT_C:
		USB_EPcConfig0 = (USB_EPcConfig0 & ~USB_EPcConfig0_INxOUT) |
							(direction & USB_EPcConfig0_INxOUT);
		break;
	case USB_ENDPOINT_D:
		USB_EPdConfig0 = (USB_EPdConfig0 & ~USB_EPdConfig0_INxOUT) |
							(direction & USB_EPdConfig0_INxOUT);
		break;
	}
}

/*************************************************************
	UsbGetEndpointDirection()

	description:
		Get endpoint direction.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		0xFF	: Error
		USB_EP_IN
		USB_EP_OUT

**************************************************************/
unsigned char UsbGetEndpointDirection(unsigned char number){

	unsigned char	direction;

	direction = 0xFF;

	switch(number){
	case USB_ENDPOINT_A:
		direction = (USB_EPaConfig0 & USB_EPaConfig0_INxOUT);
		break;
	case USB_ENDPOINT_B:
		direction = (USB_EPbConfig0 & USB_EPbConfig0_INxOUT);
		break;
	case USB_ENDPOINT_C:
		direction = (USB_EPcConfig0 & USB_EPcConfig0_INxOUT);
		break;
	case USB_ENDPOINT_D:
		direction = (USB_EPdConfig0 & USB_EPdConfig0_INxOUT);
		break;
	}

	return direction;
}

/*************************************************************
	UsbSetEndpointMode()

	description:
		Set endpoint mode.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		mode	: Endpoint mode
					USB_EP_ISO
					USB_EP_ISO_CRCMODE
					USB_EP_BULK_INTR
	return		: None

**************************************************************/
void UsbSetEndpointMode(unsigned char number, unsigned char mode){

	switch(number){
	case USB_ENDPOINT_A:
		USB_EPaConfig1 = mode;
		break;
	case USB_ENDPOINT_B:
		USB_EPbConfig1 = mode;
		break;
	case USB_ENDPOINT_C:
		USB_EPcConfig1 = mode;
		break;
	case USB_ENDPOINT_D:
		USB_EPdConfig1 = mode;
		break;
	}
}

/*************************************************************
	UsbGetEndpointMode()

	description:
		Get endpoint mode.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		0xFF	: Error
		USB_EP_ISO
		USB_EP_ISO_CRCMODE
		USB_EP_BULK_INTR

**************************************************************/
unsigned char UsbGetEndpointMode(unsigned char number){

	unsigned char	mode;

	switch(number){
	case USB_ENDPOINT_A:
		mode = USB_EPaConfig1 & (USB_EPaConfig1_ISO | USB_EPaConfig1_ISOCRCmode);
		break;
	case USB_ENDPOINT_B:
		mode = USB_EPbConfig1 & (USB_EPbConfig1_ISO | USB_EPbConfig1_ISOCRCmode);
		break;
	case USB_ENDPOINT_C:
		mode = USB_EPcConfig1 & (USB_EPcConfig1_ISO | USB_EPcConfig1_ISOCRCmode);
		break;
	case USB_ENDPOINT_D:
		mode = USB_EPdConfig1 & (USB_EPdConfig1_ISO | USB_EPdConfig1_ISOCRCmode);
		break;
	}

	return mode;
}

/*************************************************************
	UsbSetEndpointToggleMode()

	description:
		Set endpoint toggle mode.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
		mode	: Endpoint toggle mode
					USB_EP_TOGGLE_NORMAL
					USB_EP_TOGGLE_ALWAYS
	return		: None

**************************************************************/
void UsbSetEndpointToggleMode(unsigned char number, unsigned char mode){

	switch(number){
	case USB_ENDPOINT_A:
		USB_EPaConfig0 = (USB_EPaConfig0 & ~USB_EPaConfig0_ToggleMode) | mode;
		break;
	case USB_ENDPOINT_B:
		USB_EPbConfig0 = (USB_EPbConfig0 & ~USB_EPbConfig0_ToggleMode) | mode;
		break;
	case USB_ENDPOINT_C:
		USB_EPcConfig0 = (USB_EPcConfig0 & ~USB_EPcConfig0_ToggleMode) | mode;
		break;
	case USB_ENDPOINT_D:
		USB_EPdConfig0 = (USB_EPdConfig0 & ~USB_EPdConfig0_ToggleMode) | mode;
		break;
	}
}


/*************************************************************
	UsbGetEndpointToggleMode()

	description:
		Get endpoint toggle mode.

	argument	:
		number	: Endpoint number
					USB_ENDPOINT_A
					USB_ENDPOINT_B
					USB_ENDPOINT_C
					USB_ENDPOINT_D
	return		:
		0xFF	: Error
		USB_EP_TOGGLE_NORMAL
		USB_EP_TOGGLE_ALWAYS

**************************************************************/
unsigned char UsbGetEndpointToggleMode(unsigned char number){

	unsigned char		mode;

	switch(number){
	case USB_ENDPOINT_A:
		mode = (USB_EPaConfig0 & USB_EPaConfig0_ToggleMode);
		break;
	case USB_ENDPOINT_B:
		mode = (USB_EPbConfig0 & USB_EPbConfig0_ToggleMode);
		break;
	case USB_ENDPOINT_C:
		mode = (USB_EPcConfig0 & USB_EPcConfig0_ToggleMode);
		break;
	case USB_ENDPOINT_D:
		mode = (USB_EPdConfig0 & USB_EPdConfig0_ToggleMode);
		break;
	}

	return mode;
}
#endif

/*************************************************************
	UsbSetFIFOArea()

	description:
		FIFO size is allocated.

	argument	:
		map		: FIFO map
	return		: None

**************************************************************/
void UsbSetFIFOArea(unsigned short *map){

	// save Descriptor top address
	DescriptorAddress = map[0];

	USB_DescAdrsH = ((unsigned char *)&map[0])[1];
	USB_DescAdrsL = ((unsigned char *)&map[0])[0];
#ifdef	FOR_BULK_TRANSFER
	USB_EPaStartAdrsH = ((unsigned char *)&map[1])[1];
	USB_EPaStartAdrsL = ((unsigned char *)&map[1])[0];
	USB_EPbStartAdrsH = ((unsigned char *)&map[2])[1];
	USB_EPbStartAdrsL = ((unsigned char *)&map[2])[0];
	USB_EPcStartAdrsH = ((unsigned char *)&map[3])[1];
	USB_EPcStartAdrsL = ((unsigned char *)&map[3])[0];
	USB_EPdStartAdrsH = ((unsigned char *)&map[4])[1];
	USB_EPdStartAdrsL = ((unsigned char *)&map[4])[0];
#endif
}

#ifdef	FOR_TEST_MODE
/*************************************************************
	UsbGetFIFOArea()

	description:
		Read FIFO map.

	argument	:
		map		: FIFO map
	return		: None

**************************************************************/
void UsbGetFIFOArea(unsigned short *map){

	map[0] = DescriptorAddress;

	((unsigned char *)&map[1])[1] = USB_EPaStartAdrsH;
	((unsigned char *)&map[1])[0] = USB_EPaStartAdrsL;
	((unsigned char *)&map[2])[1] = USB_EPbStartAdrsH;
	((unsigned char *)&map[2])[0] = USB_EPbStartAdrsL;
	((unsigned char *)&map[3])[1] = USB_EPcStartAdrsH;
	((unsigned char *)&map[3])[0] = USB_EPcStartAdrsL;
	((unsigned char *)&map[4])[1] = USB_EPdStartAdrsH;
	((unsigned char *)&map[4])[0] = USB_EPdStartAdrsL;
}
#endif

/*************************************************************
	UsbWriteDescriptor()

	description:
		Write descriptor data to FIFO.

	argument	:
		buffer	: FIFO map
		offset	: offset from Descriptor Area Top address
		length	: data size
	return		: None

**************************************************************/
void UsbWriteDescriptor(unsigned char *buffer, unsigned short offset, unsigned short length){

	unsigned short		i;
	unsigned short		address;

	// Descriptor FIFO address
	address = DescriptorAddress + offset;

	// Set descriptor address
	USB_DescAdrsH = ((unsigned char *)&address)[1];
	USB_DescAdrsL = ((unsigned char *)&address)[0];

	// write to FIFO
	for(i = 0; i < length; i++){
		USB_DescDoor = *buffer++;
	}
}

#ifndef	FOR_USB_MINIMUM_API
/*************************************************************
	UsbReadDescriptor()

	description:
		Read descriptor data to FIFO.

	argument	:
		buffer	: FIFO map
		offset	: offset from Descriptor Area Top address
		length	: data size
	return		: None

**************************************************************/
void UsbReadDescriptor(unsigned char *buffer, unsigned short offset, unsigned short length){

	unsigned short		i;
	unsigned short		address;

	// Descriptor FIFO address
	address = DescriptorAddress + offset;

	// Set descriptor address
	USB_DescAdrsH = ((unsigned char *)&address)[1];
	USB_DescAdrsL = ((unsigned char *)&address)[0];

	// write to FIFO
	for(i = 0; i < length; i++){
		*buffer++ = USB_DescDoor;
	}
}
#endif

/*************************************************************
	UsbReplyDescriptor()

	description:
		Reply descripor.

	argument	:
		offset	: offset from Descriptor Area Top address
		length	: data size
	return		: None

**************************************************************/
void UsbReplyDescriptor(unsigned short offset, unsigned short length){

	unsigned short		address;

	// Descriptor FIFO address
	address = DescriptorAddress + offset;

	// Set descriptor address
	USB_DescAdrsH = ((unsigned char *)&address)[1];
	USB_DescAdrsL = ((unsigned char *)&address)[0];

	// Set reply size
	USB_DescSizeH = ((unsigned char *)&length)[1];
	USB_DescSizeL = ((unsigned char *)&length)[0];

	// Set Reply Descriptor
	USB_EP0Control |= USB_EP0Control_ReplyDescriptor;
}

/*************************************************************
	UsbRemainDescriptor()

	description:
		Remain descripor.

	argument	: None
	return		: None

**************************************************************/
unsigned short UsbRemainDescriptor(void){

	unsigned short		length;

	// Read reply size
	((unsigned char *)&length)[1] = USB_DescSizeH;
	((unsigned char *)&length)[0] = USB_DescSizeL;

	return length;
}

/*************************************************************
	UsbGetUSBState()

	description:
		Get USB Device state.

	argument	: None
	return		:
		USB_DEFAULT_STATE
        USB_ADDRESS_STATE
        USB_CONFIGURED_STATE

**************************************************************/
unsigned char UsbGetUSBState(void){

	return g_UsbState;
}

/*************************************************************
	ActiveUSBIP()

	description:
		Clock active.

	argument	: None
	return		: None

**************************************************************/
void ActiveUSBIP(void){


	if(g_UsbSnoozeFlag == 1){
		// Active
		g_Config.activeFunc();
		g_UsbSnoozeFlag = 0;
	}
}

#ifndef	FOR_SNOOZE_OFF
/*************************************************************
	SnoozeUSBIP()

	description:
		Clock snooze.

	argument	: None
	return		: None

**************************************************************/
void SnoozeUSBIP(void){

	if(g_UsbSnoozeFlag == 0){
		// Snooze
		g_Config.snoozeFunc();

		g_UsbSnoozeFlag = 1;
	}
}
#endif

/*************************************************************
	InitializeModule()

	description:
		Reset Initialize for module.

	argument	: None
	return		: None

**************************************************************/
static void InitializeModule(void){

	int			i;

	// Set Auto force NAK for EP0
	USB_EP0ControlOUT |= USB_EP0ControlOUT_AutoForceNAK;
	// Clear Reply Descriptor
	USB_EP0Control &= ~USB_EP0Control_ReplyDescriptor;

	// Endpoint0 Enable interrupt
	USB_EP0IntEnb = (USB_EP0IntEnb_EnINTranACK | USB_EP0IntEnb_EnOUTTranACK |
					 USB_EP0IntEnb_EnINTranErr | USB_EP0IntEnb_EnOUTTranErr);

	// Disable Endpoint
	USB_EPaConfig0 &= ~USB_EPaConfig0_EnEndPoint;
	USB_EPbConfig0 &= ~USB_EPbConfig0_EnEndPoint;
	USB_EPcConfig0 &= ~USB_EPcConfig0_EnEndPoint;
	USB_EPdConfig0 &= ~USB_EPdConfig0_EnEndPoint;

	// Clear Receive EP0 Setup
	USB_MainIntStat = USB_MainIntStat_RcvEP0SETUP;

	InterruptDisableUSB();
	// Clear Interrupt save factor
	g_MainInterruptFactor = 0;
	g_SIEInterruptFactor  = 0;
	g_EPrInterruptFactor  = 0;
	g_FIFOInterruptFactor = 0;
	g_EP0InterruptFactor  = 0;
#ifdef	FOR_BULK_TRANSFER
	g_EPaInterruptFactor  = 0;
	g_EPbInterruptFactor  = 0;
	g_EPcInterruptFactor  = 0;
	g_EPdInterruptFactor  = 0;
#endif

	g_ManagementFlag.USBReqRcvd			= 0;
	g_ManagementFlag.EnbRemoteWakeup	= 0;
#ifdef	FOR_TEST_MODE
	g_ManagementFlag.TestMode			= 0;
#endif
	g_ManagementFlag.Attach				= 0;
	g_ManagementFlag.Detach				= 0;
	g_ManagementFlag.Reset				= 0;
	g_ManagementFlag.Suspend			= 0;
	g_ManagementFlag.Resume				= 0;
	g_ManagementFlag.NonJ				= 0;
	InterruptEnableUSB();


	// Clear Process state
	g_EP0State = EP0_IDLE;
#ifdef	FOR_BULK_TRANSFER
	g_EPxState[0] = USB_EPX_IDLE;
	g_EPxState[1] = USB_EPX_IDLE;
	g_EPxState[2] = USB_EPX_IDLE;
	g_EPxState[3] = USB_EPX_IDLE;
#endif

	// Set FIFO area
	UsbSetFIFOArea(&g_UsbFifoMap[0][0]);

	// All FIFO Clear
	UsbFIFOAllClear();

	g_UsbState = USB_DEFAULT_STATE;			// USB Device state
	g_EP0State = EP0_IDLE;					// Endpoint0 state
	g_EP0Result = EP0_NO_RESULT;			// Endpoint0 result

#ifdef	FOR_BULK_TRANSFER
	for(i = 0; i < 4; i++){
		g_EPxState[i]  = USB_EPX_IDLE;		// Endpoint x state
		g_EPxResult[i] = USB_EPX_NO_RESULT;	// Endpoint x result
	}
#endif

#ifdef	FOR_CLASS_REQUEST
	// Registry Reset function
	// Call Reset function
	g_Config.resetFunc();
#endif

	g_ManagementFlag.Init = 1;
}

